import cn.estsh.i3plus.pojo.mes.bean.MesRawPackage
import cn.estsh.i3plus.pojo.mes.bean.MesRawPackageDetail
import cn.estsh.i3plus.pojo.mes.bean.MesRawPackagePart
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesRawPackageDetailRepository
import cn.estsh.i3plus.pojo.mes.repository.MesRawPackagePartRepository
import cn.estsh.i3plus.pojo.mes.repository.MesRawPackageRepository
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.util.StringUtils

/**
 * @Description : 加料防错
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/13 13:03
 * @Modify:
 * */
class MesToPiscesRawPartCharging {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesToPiscesRawPartCharging.class)

    @Autowired
    private MesRawPackageRepository mesRawPackageRepository;

    @Autowired
    private MesRawPackageDetailRepository mesRawPackageDetailRepository;

    @Autowired
    private MesRawPackagePartRepository mesRawPackagePartRepository;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        Map<Long, MesRawPackage> mesRawPackageMap = new HashMap<>();
        Map<Long, MesRawPackageDetail> mesRawPackageDetailMap = new HashMap<>();
        Map<Long, MesRawPackagePart> mesRawPackagePartMap = new HashMap<>();
        for (Map<String, Object> rowMap : srcData) {
            //原材料容器
            if (!StringUtils.isEmpty(rowMap.get("package_id"))) {
                long id = Long.parseLong(String.valueOf(rowMap.get("package_id")));
                if (Objects.isNull(mesRawPackageMap) || !mesRawPackageMap.containsKey(id)) {
                    mesRawPackageMap.put(id, mesRawPackageRepository.getById(id))
                }
                MesRawPackage mesRawPackage = mesRawPackageMap.get(id)
                if (!Objects.isNull(mesRawPackage)) {
                    rowMap.put("PACKAGE_NAME", mesRawPackage.getName())
                    rowMap.put("PACKAGE_FID", mesRawPackage.getFid())
                } else {
                    LOGGER.info("原材料容器id：{}，信息不存在", String.valueOf(rowMap.get(rowMap.get("package_id"))))
                }
            }else{
                rowMap.put("PACKAGE_NAME", null)
                rowMap.put("PACKAGE_FID", null)
            }
            //防错配置明细
            if (!StringUtils.isEmpty(rowMap.get("package_detail_id"))) {
                long id = Long.parseLong(String.valueOf(rowMap.get("package_detail_id")));
                if (Objects.isNull(mesRawPackageDetailMap) || !mesRawPackageDetailMap.containsKey(id)) {
                    mesRawPackageDetailMap.put(id, mesRawPackageDetailRepository.getById(id))
                }
                MesRawPackageDetail mesRawPackageDetail = mesRawPackageDetailMap.get(id)
                if (!Objects.isNull(mesRawPackageDetail)) {
                    rowMap.put("PACKAGE_DETAIL_FID", mesRawPackageDetail.getFid())
                } else {
                    LOGGER.info("原材料容器明细id：{}，信息不存在", String.valueOf(rowMap.get(rowMap.get("package_detail_id"))))
                }
            }else{
                rowMap.put("PACKAGE_DETAIL_FID", null)
            }
            //原材料容器零件关系表
            if (!StringUtils.isEmpty(rowMap.get("package_part_id"))) {
                long id = Long.parseLong(String.valueOf(rowMap.get("package_part_id")));
                if (Objects.isNull(mesRawPackagePartMap) || !mesRawPackagePartMap.containsKey(id)) {
                    mesRawPackagePartMap.put(id, mesRawPackagePartRepository.getById(id))
                }
                MesRawPackagePart mesRawPackagePart = mesRawPackagePartMap.get(id)
                if (!Objects.isNull(mesRawPackagePart)) {
                    rowMap.put("PACKAGE_PART_FID", mesRawPackagePart.getFid())
                } else {
                    LOGGER.info("原材料容器明细id：{}，信息不存在", String.valueOf(rowMap.get(rowMap.get("package_part_id"))))
                }
            }else {
                rowMap.put("PACKAGE_PART_FID", null)
            }
        }
        return srcData;
    }
}