import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderRepository
import lombok.Getter
import lombok.Setter
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate
import org.springframework.util.CollectionUtils

import javax.annotation.Resource
import java.sql.Connection

/**
 * @Description : 加工记录同步 MES2PISCES
 * @Reference :
 * @Author : wangjie
 * @CreateDate 2024/7/12 12:28
 * @Modify:
 * */
class MesProductionRecordToPisces {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesProductionRecordToPisces.class)

    @Autowired
    private MesEquipmentRepository equipmentRepository;

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Resource(name = "piscesDataSource")
    private DynamicDataSourceProxy piscesDataSource;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Getter
    @Setter
    private Connection mesConn;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        List<String> updateList = new ArrayList<>();
        List<Map<String, Object>> destData = new ArrayList<>();
        for (Map<String, Object> rowMap : srcData) {

            Map<String, Object> params = new HashMap<>();
            params.put("partNo",rowMap.get("part_no"))
            params.put("productSn",rowMap.get("product_sn"))
            String sql = "select ID from MES.TT_PC_PART_BARCODE where PART_NO = :partNo and BARCODE = :productSn order by GENERATION_TIME desc";
            List<Map<String, Object>> dataMap = queryDataTable(sql,params);

            if (!CollectionUtils.isEmpty(dataMap) && dataMap.size() >0) {
                rowMap.put("BARCODE_ID", dataMap.iterator().next().get("ID"))
                updateList.add("update mes_production_record set system_sync_status = 1 where id= "+rowMap.get("id")+" ;")
            }else{
                continue
            }

            String workOrderNo = String.valueOf(rowMap.get("work_order_no"));
            MesWorkOrder workOrderDb = getWorkOrderDb(mapper.getOrganizeCode(), workOrderNo);
            if (!Objects.isNull(workOrderDb)) {
                rowMap.put("WORK_ORDER_FID", workOrderDb.getFid())
            }

            String equipmentCode = String.valueOf(rowMap.get("equipment_code"));
            MesEquipment equipmentDb = getEquipmentDb(mapper.getOrganizeCode(), equipmentCode);
            if (!Objects.isNull(equipmentDb)) {
                rowMap.put("EQUIPMENT_ID", equipmentDb.getEquipId())
            }
            destData.add(rowMap);
        }

        //更新同步标识
        if(!CollectionUtils.isEmpty(updateList) && updateList.size() > 0){
            this.mesConn = mesDataSource.getWriteConnectionWithoutPool();
            mesDataSource.executeAsBatch(updateList,mesConn)
            mesDataSource.closeConnectionWithoutPoll(this.mesConn)
        }

        return destData;
    }

    def getProduceSnDb(String organizeCode, String productSn, String partNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productSn, "productSn", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        MesProduceSn produceSnDb = produceSnRepository.getByProperty(ddlPackBean);
        return produceSnDb;
    }

    MesWorkOrder getWorkOrderDb(String organizeCode, String workOrderNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workOrderNo, "workOrderNo", ddlPackBean);
        MesWorkOrder workOrderDb = workOrderRepository.getByProperty(ddlPackBean);
        return workOrderDb;
    }

    def getEquipmentDb(String organizeCode, String equipmentCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", ddlPackBean);
        MesEquipment equipmentDb = equipmentRepository.getByProperty(ddlPackBean);
        return equipmentDb;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(piscesDataSource.getDataSource())
        List<Map<String, Object>> dataMap = namedJdbcTemplate.queryForList(sql, params)
        return dataMap;
    }

}