import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.mes.bean.MesCrossOrganizeCfg
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.IMesCrossOrganizeCfgRepository
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate
import org.springframework.util.CollectionUtils

import javax.annotation.Resource

/**
 * @Description : 跨工厂防错
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/22 11:03
 * @Modify:
 * */
class PiscesToMesProductSn {

    public static final Logger LOGGER = LoggerFactory.getLogger(PiscesToMesProductSn.class)

    @Autowired
    private IMesCrossOrganizeCfgRepository mesCrossOrganizeCfgRepository;

    @Resource(name = "pisces1DataSource")
    private DynamicDataSourceProxy pisces1DataSource;

    def readData(MesInterfaceDataMapper mapper) throws Exception {
        String organizeCode = mapper.getOrganizeCode();
        List<MesCrossOrganizeCfg> crossOrganizeCfgList = mesCrossOrganizeCfgRepository.findByHqlWhere(DdlPackBean.getDdlPackBean(organizeCode));
        if (CollectionUtils.isEmpty(crossOrganizeCfgList)) {
            LOGGER.info("未查询到跨工厂配置");
            return new ArrayList<>();
        }
        List<Map<String, Object>> srcData = new ArrayList<>();
        //循环处理数据
        for (MesCrossOrganizeCfg crossOrganizeCfg : crossOrganizeCfgList) {
            Map<String, Object> params = new HashMap<>();
            params.put("srcOrganizeCode", crossOrganizeCfg.getSrcOrganizeCode())
            params.put("partNo", crossOrganizeCfg.getPartNo())
            String sql = "select top "+mapper.getSrcGetLimit()+" * from " + mapper.getSrcTableName() + " where PLANT = :srcOrganizeCode and PART_NO = :partNo and " + mapper.getSrcWhere() + " order by "+mapper.getSrcOrderBy()
            //查询待同步的数据
            List<Map<String, Object>> dataList = queryDataTable(sql, params);
            if (!CollectionUtils.isEmpty(dataList)) {
                srcData.addAll(dataList);
            }
        }
        return srcData;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(pisces1DataSource.getDataSource())
        return namedJdbcTemplate.queryForList(sql, params);
    }
}