import cn.estsh.i3plus.ext.mes.api.busi.IMesInputDefectRecordService
import cn.estsh.i3plus.platform.common.tool.TimeTool
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.mes.bean.MesDefectType
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesDefectTypeRepository
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderToSapRepository
import lombok.Getter
import lombok.Setter
import org.apache.commons.collections.CollectionUtils
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate

import javax.annotation.Resource
import java.sql.Connection

/**
 * QMS同步MES_CCSC任务检测结果
 * @Author : gsz
 * @CreateDate 2024/5/23 9:58
 */
class sapToMesCcscTaskResult {

    public static final Logger LOGGER = LoggerFactory.getLogger(sapToMesCcscTaskResult.class)

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;
    @Autowired
    private MesWorkOrderToSapRepository mesWorkOrderToSapRDao;

    @Resource(name = "yfasDataSource")
    private DynamicDataSourceProxy yfasDataSource;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private IMesInputDefectRecordService mesInputDefectRecordService;

    @Autowired
    private MesDefectTypeRepository mesDefectTypeRepository;

    @Getter
    @Setter
    private Connection mesConn;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        Map<String, Map<String, Object>> destMapData = new HashMap<>();
        LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 ---------------")

        List<Map<String, Object>> destData = new ArrayList<>();
        List<String> insertSqlList = new ArrayList<>();
        MesDefectType mesDefect = null;
        for (Map<String, Object> rowMap : srcData) {
            String organizeCode = String.valueOf(rowMap.get("WERKS"));
            String taskNo = rowMap.get("task_no").toString();
            String taskResult = rowMap.get("task_result").toString(); //1:合格 2:不合格

            Map<String, Object> sqlParams = new HashMap<>(6)
            sqlParams.put("organizeCode", organizeCode);
            sqlParams.put("task_no", taskNo);
            StringBuffer sql = new StringBuffer("select * from  mes_ccsc_task where " +
                    " organize_code=:organizeCode and task_no=:task_no ; ");
            List<Map<String, Object>> resultMap = this.queryDataTable(sql.toString(), sqlParams)

            insertSqlList.add("update mes_ccsc_task set  task_result = '"+taskResult+ "'  " +
                    "  where organize_code= '"+organizeCode+"' and task_no= '"+taskNo+"'; ");

            LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 60-{}-------",taskResult)

            if (resultMap.size() > 0) {
                sqlParams.put("work_order_no", resultMap.get(0).get("work_order_no"));
                String part_no = resultMap.get(0).get("part_no")
                sqlParams.put("part_no", part_no);
                StringBuffer sqlWork = new StringBuffer("select * from  mes_work_order where " +
                        " organize_code=:organizeCode and work_order_no=:work_order_no and part_no=:part_no ; ");
                List<Map<String, Object>> resultWorkMap = this.queryDataTable(sqlWork.toString(), sqlParams)

                if (resultWorkMap.size() > 0) {
                    String work_center_code = resultWorkMap.get(0).get("work_center_code")
                    String work_cell_code = resultWorkMap.get(0).get("work_cell_code")
                    long workOrderId = Long.valueOf(resultWorkMap.get(0).get("id")+"")

                    LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 70{}",work_center_code)
                    String logUpdateSql="update mes_comunication_log set  is_deleted = 1 " +
                            " where organize_code= '"+organizeCode+"' and work_center_code= '"+work_center_code+"' " +
                            " and work_cell_code= '"+work_cell_code+"' and parameter_name= '"+part_no+"'; "
                    insertSqlList.add(logUpdateSql);
                    //更新工单CCSC状态
                    String workOrderUpdate = "";
                    if (taskResult.equals("2")) { //1:合格 2:不合格
                        String insertSql = String.format(" INSERT INTO mes_comunication_log " +
                                "(`parameter_name`,`parameter_desc`,`value`,`status`,`work_cell_code`,`work_center_code`, " +
                                "`id`,`organize_code`,`create_date_time`,`create_user` )" +
                                "VALUES('%s', '%s','%s','%s','%s','%s','%s','%s','%s','%s') ; "
                                , part_no, part_no, part_no, 1, work_cell_code, work_center_code,
                                snowflakeIdMaker.nextId(), organizeCode, TimeTool.getNowTime(true), "CCSC_SAP");
                        insertSqlList.add(insertSql);
                        //查询所有条码信息
                        List<MesProduceSn> mesProduceSnList = getMesProduceSn(organizeCode, work_center_code, work_cell_code, part_no)
                        if(CollectionUtils.isNotEmpty(mesProduceSnList)){
                            if(Objects.isNull(mesDefect)){
                                mesDefect = getMesDefect(organizeCode)
                            }
                            //更新所有条码为可疑品 生成可疑品记录
                            for (MesProduceSn mesProduceSn : mesProduceSnList) {
                                mesInputDefectRecordService.savePartInspection(mesProduceSn, "JOB",mesDefect,"A01")
                            }
                        }
                        workOrderUpdate = "update mes_work_order set  ccsc_status = 20,work_order_status=40 where id= "+workOrderId+"; "
                    }else{
                        workOrderUpdate = "update mes_work_order set  ccsc_status = 10 where id= "+workOrderId+"; "
                    }
                    insertSqlList.add(workOrderUpdate);

                }
            }
            destData.add(rowMap);

        }
        //更新同步标识
        this.mesConn = mesDataSource.getWriteConnectionWithoutPool();
        mesDataSource.executeAsBatch(insertSqlList, mesConn)
        mesDataSource.closeConnectionWithoutPoll(this.mesConn)

        //更新同步标识
//        this.mesConn = yfasDataSource.getWriteConnectionWithoutPool();
//        yfasDataSource.executeAsBatch(updateSqlList,mesConn)
//        yfasDataSource.closeConnectionWithoutPoll(this.mesConn)
        return destData;
    }

    private MesDefectType getMesDefect(String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode)
        DdlPreparedPack.getStringEqualPack("CCSC", "defectTypeCode", packBean);
        return mesDefectTypeRepository.getByProperty(packBean)
    }

    private List<MesProduceSn> getMesProduceSn(String organizeCode, String work_center_code, String work_cell_code, String part_no) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode)
        DdlPreparedPack.getStringEqualPack(work_center_code, "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(work_cell_code, "workCellCode", packBean);
        DdlPreparedPack.getStringEqualPack(part_no, "partNo", packBean);
        return mesProduceSnRepository.findByHqlWhere(packBean)
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(mesDataSource.getDataSource())

        List<Map<String, Object>> dataMap = namedJdbcTemplate.queryForList(sql, params)

        return dataMap;
    }
}