package cn.estsh.i3plus.ext.mes.apiservice.controller.busi;

import cn.estsh.i3plus.ext.mes.api.base.*;
import cn.estsh.i3plus.ext.mes.api.busi.IMaximoService;
import cn.estsh.i3plus.ext.mes.pojo.model.MaximoParamModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesShift;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShiftRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/white/maximo")
@Api(description = "白名单接口maximo")
public class MesWhiteListController extends MesBaseController {

    @Autowired
    private IMaximoService maximoService;

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesCcscTaskService mesCcscTaskService;
    @Autowired
    private IMesWorkOrderService workOrderService;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;
    @Autowired
    private MesShiftRepository mesShiftRepository;
    @Autowired
    private IMesConfigService configService;

    @Autowired
    private MesPartInspectionRepository partInspectionRepository;

    @Autowired
    private IMesQmsScrapService qmsScrapService;
    @Autowired
    private IMesQmsSuspiciousService qmsSuspiciousService;

    @Autowired
    private IMesCrossOrganizeCfgService mesCrossOrganizeCfgService;

    @PostMapping(value = "/meter/main-data")
    @ApiOperation(value = "Maximo仪表主数据", notes = "Maximo仪表主数据")
    public Map doMeterMainData(@RequestBody MaximoParamModel paramModel) {
        // 数据校验
        ValidatorBean.checkNotNull(paramModel.getMeterName(), "meterName不能为空");
        ValidatorBean.checkNotNull(paramModel.getAssetNum(), "assetNum不能为空");
        ValidatorBean.checkNotNull(paramModel.getBaseMeasureUnitId(), "baseMeasureUnitId不能为空");
        ValidatorBean.checkNotNull(paramModel.getSiteId(), "siteId不能为空");
        Map resultMap = maximoService.doMeterMainData(paramModel);

        LOGGER.info("Maximo仪表主数据 返回 resultMap:{}", resultMap);

        return resultMap;

    }

    @PostMapping(value = "/asset/main-data")
    @ApiOperation(value = "Maximo设备主数据", notes = "Maximo设备主数据")
    public Map doAssetMainData(@RequestBody MaximoParamModel paramModel) {
        // 数据校验
        ValidatorBean.checkNotNull(paramModel.getAssetNum(), "assetNum不能为空");
        ValidatorBean.checkNotNull(paramModel.getDescription(), "description不能为空");
        ValidatorBean.checkNotNull(paramModel.getSiteId(), "siteId不能为空");
        ValidatorBean.checkNotNull(paramModel.getStatus(), "status不能为空");
        ValidatorBean.checkNotNull(paramModel.getFailureCode(), "failureCode不能为空");
        Map resultMap = maximoService.doAssetMainData(paramModel);

        LOGGER.info("Maximo设备主数据 返回 resultMap:{}", resultMap);

        return resultMap;
//        return ResultBean.success().setResultMap(resultMap).setSuccess((Boolean) resultMap.get("success"));

    }

    @GetMapping(value = "/ccscTest")
    @ApiOperation(value = "ccsc", notes = " ")
    public ResultBean doCcscTest(String organizeCode) {
        List<MesWorkOrder> mesWorkOrderList = mesWorkOrderService.queryMesWorkOrderProcessToSap(organizeCode);
        if (!CollectionUtils.isEmpty(mesWorkOrderList)) {
            mesCcscTaskService.doCcscTaskByMesWorkOrder(organizeCode, mesWorkOrderList);
        }
        return ResultBean.success().setSuccess(true);
    }

    @GetMapping(value = "/doMesCrossOrganizeCfg")
    @ApiOperation(value = "跨工厂防错JOB")
    public ResultBean doMesCrossOrganizeCfg(String organizeCode, Integer pageSize) {
        //统计数据
        try {
            mesCrossOrganizeCfgService.doMesCrossOrganizeCfg(organizeCode, pageSize, "job");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
        return ResultBean.success("执行成功");
    }

    @GetMapping("/reportWorkByPreDayJob")
    @ApiOperation(value = "job测试")
    public ResultBean queryReworkTaskByPager(String organizeCode) {

            //遍历系统参数指定的产线获取对应班次，找到早班的开始时间，
            String workCenterCodes = configService.getCfgValue(organizeCode, "MES_PCN_WORK_CENTER_NO");
            List<String> workCenterCodeList = Arrays.asList(workCenterCodes.split(","));
            String userName = "REPORT_PRE_DAY_JOB";

            for (String workCenterCode : workCenterCodeList) {
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", packBean);
                List<MesShift> mesShiftList = mesShiftRepository.findByHqlWhere(packBean);
                //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
                List<MesProductionRecord> mesProductionRecordList = workOrderService.getPreDayReportMesProductionRecord(organizeCode, mesShiftList, workCenterCode);
                if (CollectionUtils.isEmpty(mesProductionRecordList)) {
                    continue;
                }

                //3.插入生产工单表
                List<MesWorkOrder> mesWorkOrders = workOrderService.insertMesWorkOrder(mesProductionRecordList, mesShiftList, organizeCode, userName);
                if (CollectionUtils.isEmpty(mesWorkOrders)) {
                    continue;
                }
                //4.根据产线+物料产生的工单报工
                try {
                    for (MesWorkOrder mesWorkOrder : mesWorkOrders) {
                        List<MesProductionRecord> mesProductionRecords = mesProductionRecordList.stream().filter(
                                w -> w.getWorkCenterCode().equalsIgnoreCase(mesWorkOrder.getWorkCenterCode())
                                        && w.getPartNo().equalsIgnoreCase(mesWorkOrder.getPartNo())).collect(Collectors.toList());
                        workOrderService.doPcnJobProductReport(mesProductionRecords, mesShiftList, organizeCode, userName, mesWorkOrder);
                    }
                    for (MesProductionRecord mesProductionRecord : mesProductionRecordList) {
                        mesProductionRecord.setReportStatus(20);
                        ConvertBean.serviceModelUpdate(mesProductionRecord, userName);
                    }
                    productionRecordRao.saveAll(mesProductionRecordList);

                } catch (ImppBusiException imppException) {
                    return ResultBean.fail(imppException);
                } catch (Exception e) {
                    return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
                }
            }
            return ResultBean.success();
    }

    @GetMapping(value = "/qmsScrap")
    @ApiOperation(value = "qmsScrap", notes = " ")
    public ResultBean doQmsScrapTest(String organizeCode) {
        //获取前一天的所有ncStatus=40报废状态下可疑品缺陷
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue(), "inspectionStatus", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.SCRAP.getValue(), "ncStatus", packBean);

        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), "systemSyncStatus", packBean);
        DdlPreparedPack.timeBuilder(
                new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(), 1)) + " 00:00:00",
                TimeTool.getToday() + " 00:00:00", "createDatetime", packBean, true);

        List<MesPartInspection> mesPartInspectionList = partInspectionRepository.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(mesPartInspectionList)) {
            qmsScrapService.doQmsScrapByPartInspection(mesPartInspectionList,organizeCode);

            for (MesPartInspection mesPartInspection : mesPartInspectionList) {
                mesPartInspection.setSystemSyncStatus(1);
                mesPartInspection.setSystemSyncDatetime(TimeTool.getNowTime(true));
            }
            partInspectionRepository.saveAll(mesPartInspectionList);
        }
        return ResultBean.success().setSuccess(true);
    }
    @GetMapping(value = "/qmsSuspicious")
    @ApiOperation(value = "qmsSuspicious", notes = " ")
    public ResultBean doQmsSuspiciousTest(String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
//        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), "systemSyncStatus", packBean);
        DdlPreparedPack.timeBuilder(
                new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(), 1)) + " 00:00:00",
                TimeTool.getToday() + " 00:00:00", "createDatetime", packBean, true);

        List<MesPartInspection> mesPartInspectionList = partInspectionRepository.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(mesPartInspectionList)) {
            qmsSuspiciousService.doQmsSuspiciousByPartInspection(mesPartInspectionList,organizeCode);
        }
        return ResultBean.success().setSuccess(true);
    }
}