package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesQmsSuspiciousService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionRepository;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @Description : MES可疑品汇总
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/7/3 15:58
 * @Modify:
 **/
@DisallowConcurrentExecution
@ApiOperation("MES可疑品汇总")
public class MesQmsSuspiciousJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesQmsSuspiciousJob.class);

    private static final long serialVersionUID = 5078389675215031222L;

    @Autowired
    private MesPartInspectionRepository partInspectionRepository;

    @Autowired
    private IMesQmsSuspiciousService qmsSuspiciousService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesQmsSuspiciousJob() {
        super(MesQmsSuspiciousJob.class, "MES可疑品汇总");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("MES可疑品汇总job参数为空，请检查参数");
            }
            String[] organizeCodeList = jobParam.split(",");
            LOGGER.info("MES可疑品汇总 -----start");

            for (String organizeCode : organizeCodeList) {
                //获取前一天的所有ncStatus=40报废状态下可疑品缺陷
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                //按天执行前一天的所有可疑记录
//                DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), "systemSyncStatus", packBean);
                DdlPreparedPack.timeBuilder(
                        new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(), 1)) + " 00:00:00",
                        TimeTool.getToday() + " 00:00:00", "createDatetime", packBean, true);

                List<MesPartInspection> mesPartInspectionList = partInspectionRepository.findByHqlWhere(packBean);
                if (!CollectionUtils.isEmpty(mesPartInspectionList)) {
                    qmsSuspiciousService.doQmsSuspiciousByPartInspection(mesPartInspectionList,organizeCode);

                    for (MesPartInspection mesPartInspection : mesPartInspectionList) {
                        mesPartInspection.setSystemSyncStatus(1);
                        mesPartInspection.setSystemSyncDatetime(TimeTool.getNowTime(true));
                    }
                    partInspectionRepository.saveAll(mesPartInspectionList);
                }
            }
            LOGGER.info("MES可疑品汇总 ----- end");

        } catch (Exception e) {
            LOGGER.error("MES可疑品汇总 e:{}", e.toString());
        }
    }

}
