package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesBomService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesBomDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.model.MesBomTreeModel;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesBom;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesBomService extends BaseMesService<MesBom> implements IMesBomService {

    @Autowired
    private IMesBomDao mesBomDao;

    protected void setPackQueryBean(MesBom bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getBomVersion(), "bomVersion", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public List<String> findBomVersionByPartNo(MesBom mesBom) {
        return mesBomDao.findBomVersionByPartNo(mesBom.getPartNo(), mesBom.getOrganizeCode());
    }

    @Override
    public List<String> findBomCodeByPartNoAndBomVersion(MesBom mesBom) {
        return mesBomDao.findBomCodeByPartNoAndBomVersion(mesBom.getPartNo(), mesBom.getBomVersion(), mesBom.getOrganizeCode());
    }

    @Override
    public List queryMesDataTreeByFirstFloor(MesBom bom) {

        List resultList = new ArrayList();
        ValidatorBean.checkNotNull(bom.getPartNo(), "零件号不能为空");
        MesBomTreeModel organize = new MesBomTreeModel();

        organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        organize.setName("LML");
//        organize.setName(AuthUtil.getOrganize().getName());

        List<MesBom> bomList = baseRDao.findByProperty(new String[]{MesConstWords.IS_VALID, MesConstWords.IS_DELETED, MesConstWords.ORGANIZE_CODE, "partNo"},
                new Object[]{CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), bom.getOrganizeCode(), bom.getPartNo()});

        if (!CollectionUtils.isEmpty(bomList)) {
            organize.setChildTreeList(bomList);
            resultList.add(organize);
        }

        return resultList;
    }

    /**
     * 根据零件号查询所有
     *
     * @param bom 组织代码
     * @return
     */
    @Override
    public ListPager<MesBom> queryMesDataTree(MesBom bom, Pager pager) {
        List resultList = new ArrayList();
        MesBomTreeModel organize = new MesBomTreeModel();

        organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        organize.setName(AuthUtil.getOrganize().getName());

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bom.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bom.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomVersion(), "bomVersion", packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomCode(), "bomCode", packBean);
        DdlPreparedPack.getStringSmallerPack(bom.getEffStartTime(), "effStartTime", packBean);
        packBean.setOrderByStr(" order by createDatetime desc ");
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));

        List<MesBom> bomList = baseRDao.findByHqlWherePage(packBean, pager);

        recursionBomList(bomList,bom.getEffStartTime());
        organize.setChildTreeList(bomList);
        resultList.add(organize);

        return new ListPager<>(resultList, pager);
    }

    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion, String bomCode, String effStartTime) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomCode, "bomCode", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(effStartTime, "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy(null, null, ddlPackBean);
        List<MesBom> mesBoms = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】BOM代码【%s】生效时间小于【%s】对应bom信息不存在", partNo, bomVersion, bomCode, effStartTime);
        }
        return mesBoms;
    }

    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom next = mesBoms.iterator().next();
        //如果存在多个bomCode取最新的一个
        return mesBoms.stream().collect(Collectors.groupingBy(MesBom::getBomCode)).get(next.getBomCode());
    }

    @Override
    public List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }

    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails = this.findMesBomByPartNoAndBomVersion(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = MathOperation.div(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }

    /**
     * 递归查询bom
     *
     * @return
     */

    private List<MesBom> recursionBomList(List<MesBom> bomList,String effStartTime) {
        if (!CollectionUtils.isEmpty(bomList)) {
            for (MesBom bom : bomList) {
                if (Objects.equals("X", bom.getPartType())) {
                    List<MesBom> resultList = findMesBomByPartNoAndBomVersion(bom.getItemPartNo(), bom.getOrganizeCode(), bom.getBomVersion(), bom.getBomCode(), effStartTime);
                    bom.setChildTreeList(resultList);
                    recursionBomList(resultList,effStartTime);
                }
            }
        }
        return bomList;

    }


}
