package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesBtoJobService;
import cn.estsh.i3plus.ext.mes.api.base.strategy.IRoundnessStrategy;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerMessagePoint;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroupDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesCustSortInfo;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/2 11:51
 * @Modify:
 **/
@Service
@Slf4j
public class MesBtoJobServiceImpl implements IMesBtoJobService {
    @Autowired
    private MesPartProdGroupService  mesPartProdGroupService;

    @Autowired
    private MesCustomerMessagePointService mesCustomerMessagePointService;

    @Autowired
    private MesCustSoftInfoServiceImpl mesCustSoftInfoService;
    @Override
    public void doCreateBto(String organizeCode, List<String> groupCodeList) {
        //1. 根据 organizeCode 和 groupCodeList 查询零件生产组
        List<MesPartProdGroup> mesPartProdGroups = mesPartProdGroupService.findMesPartProdGroups(organizeCode, groupCodeList);
        for (MesPartProdGroup prodGroup : mesPartProdGroups) {
            //2. 根据生产组的infoPointCode 获取 过点信息MES_CUSTOMER_MESSAGE_POINT
            MesCustomerMessagePoint messagePoint = mesCustomerMessagePointService.getPointByCode(prodGroup.getCustInfoPoint(),organizeCode);
            if (messagePoint == null){
                log.info("客户的信息点{}不存在",prodGroup.getCustInfoPoint());
                continue;
            }

            List<MesPartProdGroupDetail> details = mesPartProdGroupService.findDetailByCode(prodGroup.getPartProdGroupCode(), organizeCode);
            List<String> custPartNoList = details.stream().map(MesPartProdGroupDetail::getCustPartNo).collect(Collectors.toList());
            List<MesCustSortInfo> sortInfoList = mesCustSoftInfoService.queryCustSortInfoList(prodGroup.getCustCode(), prodGroup.getCustOrganizeCode(), prodGroup.getCustInfoPoint(), organizeCode, custPartNoList);

            if (sortInfoList.isEmpty()){
                log.info("当前零件生产组:{}没有排序信息",prodGroup.getPartProdGroupName());
                continue;
            }
            //6. 根据零件生产组的RoundnessType 去做策略，现在只做 窗口时间 策略; 入参
            String roundnessService = MesExtEnumUtil.SHIPPING_GROUP_ROUND_METHOD.valueOfService(prodGroup.getRoundType());
            IRoundnessStrategy roundnessStrategy = (IRoundnessStrategy)SpringContextsUtil.getBean(roundnessService);
            roundnessStrategy.execute(prodGroup,sortInfoList, details);
        }
    }

    /**
     * 排序信息不复用的时候
     * @param infoPointCode
     * @param organizeCode
     * @param partGroupId
     * @return
     */
    private List<MesCustSortInfo> getCustSortInfoList(String infoPointCode, String organizeCode, Long partGroupId,String partGroupCode){


        return null;
    }
}
