package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesQmsSuspiciousService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesQmsSuspicious;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesReworkTaskRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description :MES可疑品汇总
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/7/3 10:28
 * @Modify:
 **/
@Service
@Slf4j
public class MesQmsSuspiciousService extends BaseMesService<MesQmsSuspicious> implements IMesQmsSuspiciousService {

    @Autowired
    private MesPartInspectionDetailRepository partInspectionDetailRDao;

    @Autowired
    private MesReworkTaskRepository mesReworkTaskRepository;

    @Override
    protected void setPackQueryBean(MesQmsSuspicious bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void doQmsSuspiciousByPartInspection(List<MesPartInspection> mesPartInspectionList, String organizeCode) {

        List<Long> idList = mesPartInspectionList.stream().map(MesPartInspection::getId).collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "pid", packBean);
        List<MesPartInspectionDetail> partInspectionDetails = partInspectionDetailRDao.findByHqlWhere(packBean);
        Map<Long, List<MesPartInspectionDetail>> idDetailListMap = partInspectionDetails.stream().collect(Collectors.groupingBy(MesPartInspectionDetail::getPid));

        Map<String, List<MesPartInspection>> workOrderPartMap = mesPartInspectionList.stream().collect(Collectors.groupingBy(MesPartInspection::getPartNo));
        //根据物料对可疑品分组
        for (Map.Entry<String, List<MesPartInspection>> stringListEntry : workOrderPartMap.entrySet()) {
            String key = stringListEntry.getKey();
            List<MesPartInspection> mesPartInspections = stringListEntry.getValue();
            MesPartInspection next = mesPartInspections.iterator().next();

            //新增一条数据到报废表中
            MesQmsSuspicious mesQmsSuspicious = new MesQmsSuspicious();
            mesQmsSuspicious.setOrganizeCode(organizeCode);
            mesQmsSuspicious.setInspectionDate(next.getInspectionDate());
            //检验数量是全部数量汇总
            double sumQty = mesPartInspections.stream().mapToDouble(MesPartInspection::getQty).sum();
            mesQmsSuspicious.setInspectionQty((int) sumQty);
            //一次合格数 nc_status=10 inspection_status=10
            double qualifiedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == 10 && m.getInspectionStatus() == 10).mapToDouble(MesPartInspection::getQty).sum();
            mesQmsSuspicious.setQualifiedQty((int) qualifiedQty);
            //不合格数 nc_status=10 inspection_status=20
            double unqualifiedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == 10 && m.getInspectionStatus() == 20).mapToDouble(MesPartInspection::getQty).sum();
            mesQmsSuspicious.setUnqualifiedQty((int) unqualifiedQty);
            //误判数量  nc_status=30 inspection_status=20
            double misjudgmentQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == 30).mapToDouble(MesPartInspection::getQty).sum();
            mesQmsSuspicious.setMisjudgmentQty((int) misjudgmentQty);
            //一次报废数  nc_status=40 inspection_status=20
            double scrappedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == 40 && m.getInspectionStatus() == 20).mapToDouble(MesPartInspection::getQty).sum();
            mesQmsSuspicious.setScrappedQty((int) scrappedQty);

            //返工合格数 nc_status=50 返工单 status=20
            Integer reworkQualified = 0;
            //返工pending（创建返工单，未处理）
            Integer reworkPending = 0;
            //返工数
            Integer reworkQty = 0;
            List<MesPartInspectionDetail> mesPartInspectionDetails = new ArrayList<>();
            for (MesPartInspection mesPartInspection : mesPartInspections) {
                if(!StringUtil.isEmpty(idDetailListMap.get(mesPartInspection.getId()))){
                    mesPartInspectionDetails.addAll(idDetailListMap.get(mesPartInspection.getId()));
                }
                //计算返工数量
                if (mesPartInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.REWORK.getValue()) {
                    MesReworkTask reworkTask = mesReworkTaskRepository.getById(mesPartInspection.getReworkTaskId());
                    if (!StringUtil.isEmpty(reworkTask)) {
                        //返工是否完成
                        if (reworkTask.getStatus() == MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue()) {
                            reworkQualified += mesPartInspection.getQty();
                        } else {
                            reworkPending += mesPartInspection.getQty();
                        }
                    } else {
                        reworkPending += mesPartInspection.getQty();
                    }
                    reworkQty+= mesPartInspection.getQty();
                }
            }
            mesQmsSuspicious.setReworkQty(reworkQty);
            mesQmsSuspicious.setReworkQualified(reworkQualified);
            mesQmsSuspicious.setReworkPending(reworkPending);
            mesQmsSuspicious.setNcPending(0);
            mesQmsSuspicious.setReworkScrapped(0);
            mesQmsSuspicious.setPartNo(key);
            mesQmsSuspicious.setPartName(next.getPartName());
            //缺陷分类，缺陷数量JOSN格式:
            //[{"Defect_Name":"缺陷01","Defect_Qty":1},{"Defect_Name":"缺陷02","Defect_Qty":2}]
            Map<String, List<MesPartInspectionDetail>> listDefectMap = mesPartInspectionDetails.stream().collect(Collectors.groupingBy(MesPartInspectionDetail::getDefectTypeName));
            List<Map<String, Object>> defectsList = new ArrayList<>();
            for (Map.Entry<String, List<MesPartInspectionDetail>> listEntry : listDefectMap.entrySet()) {
                Map<String, Object> defectMap = new HashMap<>();
                defectMap.put("Defect_Name",listEntry.getKey());
                defectMap.put("Defect_Qty",listEntry.getValue().size());
                defectsList.add(defectMap);
            }
            //缺陷分布json
            mesQmsSuspicious.setDefects(JSON.toJSONString(defectsList));
            ConvertBean.serviceModelInitialize(mesQmsSuspicious, "MesSuspiciousJob");

            baseRDao.insert(mesQmsSuspicious);
        }
    }
}
