package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.*;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesWorkOrderDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanMapUtilsExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.MesWorkOrderModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.repository.shipping.MesWorkOrderToWmsRepository;
import cn.estsh.i3plus.pojo.mes.util.DateUtilExt;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.model.wms.WmsProductModel;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.map.CaseInsensitiveMap;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesWorkOrderService extends BaseMesService<MesWorkOrder> implements IMesWorkOrderService {
    @Autowired
    private IMesWorkOrderLogService workOrderLogService;
    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private IMesPartService iMesPartService;
    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private IMesBomService mesBomService;
    @Autowired
    private IMesProductOffLineService mesProductOffLineService;
    @Autowired
    private IMesWorkOrderDao mesWorkOrderDao;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesShiftRepository mesShiftRDao;
    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;
    @Autowired
    private IMesPartProdGroupService mesPartProdGroupService;
    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;
    @Autowired
    private IMesPartSapService iMesPartSapService;
    @Autowired
    private IMesMoveRuleService mesMoveRuleService;
    @Autowired
    private MesProdRuleSortCfgRepository mesProdRuleSortCfgRDao;
    @Autowired
    private IMesProdRuleNosortCfgService mesProdRuleNosortCfgService;
    @Autowired
    private IMesPartSapService mesPartSapService;
    @Autowired
    private MesMoveRepository mesMoveRDao;
    @Autowired
    private IMesWorkOrderToWmsService mesWorkOrderToWmsService;
    @Autowired
    private MesPartSapRepository mesPartSapRDao;
    @Autowired
    private MesPartProdGroupRepository mesPartProdGroupRDao;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private MesProductOffLineRepository mesProductOffLineRDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRao;
    @Autowired
    private MesProductionRecordRepository productionRecordRao;
    @Autowired
    private MesWorkOrderLogRepository mesWorkOrderLogRepository;
    @Autowired
    private MesWorkOrderToWmsRepository mesWorkOrderToWmsRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private MesPartRepository mesPartRDao;

    @Autowired
    private IMesWorkCenterService workCenterService;

    @Autowired
    private MesWorkCenterPartRelationRepository partRelationRao;

    @Autowired
    private IMesCcscTaskService mesCcscTaskService;

    @Override
    public MesWorkOrder insert(MesWorkOrder bean) {
        //校验产线与工位的关系
        boolean relationResult = checkWorkCenterPartRelation(bean.getWorkCenterCode(), bean.getPartNo());
        if (!relationResult && (StringUtil.isEmpty(bean.getOrderFlag()) || !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(bean.getOrderFlag()))){
            MesException.throwMesBusiException(String.format("工单产线【%s】与工单零件【%s】没有维护产线零件关系",bean.getWorkCenterCode(),bean.getPartNo()));
        }
        //获取相同类型 相同标识 生产序号最大值的
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", orderPackBean);
        DdlPreparedPack.getStringEqualPack(bean.getOrderFlag(), "orderFlag", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"produceSeq"}, orderPackBean);
        MesWorkOrder result = baseRDao.getByProperty(orderPackBean);

        //插入前构造数据
        onInsertBean(bean);

        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            Long produceSeq = 0L;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = (long) Math.toIntExact(result.getProduceSeq());
            }
            MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
            BeanUtils.copyProperties(bean, copyMesWorkOrder);
            copyMesWorkOrder.setProduceSeq((long) (produceSeq + 1));
            copyMesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            ConvertBean.serviceModelInitialize(copyMesWorkOrder, bean.getCreateUser());

            baseRDao.insert(copyMesWorkOrder);
            workOrderLogService.saveMesWorkOrderLog(copyMesWorkOrder,MesExtEnumUtil.WORK_ORDER_LOG_TYPE.INSERT,copyMesWorkOrder.getQty());
            return bean;
        }else if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()){
            //生产工单-非排序手工插单，关闭该零件当天之前日期非排序工单
            List<String> partNoList = Collections.singletonList(bean.getPartNo());
            this.deleteUnSortOrder(bean.getOrganizeCode(),partNoList);
        } else if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.BTO.getValue()) {
            Long produceSeq = 0L;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = (long) Math.toIntExact(result.getProduceSeq());
            }
            bean.setProduceSeq(produceSeq + 1);
            GenSerialNoModel orderSeqSerialNoModel = new GenSerialNoModel(MesCommonConstant.ORDER_NO_BTO_SEQ);
            List<String> resultList = syncFuncService.syncSerialNo(orderSeqSerialNoModel.organizeCode(bean.getOrganizeCode()), bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
            bean.setWorkOrderSeq(resultList.get(0));
        }
        bean.setQty(bean.getQty());
        bean.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        ConvertBean.serviceModelInitialize(bean, bean.getCreateUser());

        workOrderLogService.saveMesWorkOrderLog(bean,MesExtEnumUtil.WORK_ORDER_LOG_TYPE.INSERT,bean.getQty());
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public MesWorkOrder update(MesWorkOrder bean) {
        MesWorkOrder result = baseRDao.getById(bean.getId());
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (result.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            if (!Objects.equals(bean.getQty(), result.getQty())) {
                MesException.throwMesBusiException(String.format("工单类型为【%s】 不需要修改数量",
                        MesExtEnumUtil.ORDER_TYPE.SORT.getDescription()));
            }
        } else {
            //更新工单数量
            if (!bean.getQty().equals(result.getQty()) && result.getWorkOrderStatus() > MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue()) {
                MesException.throwMesBusiException(String.format("工单状态为【%s】不允许更新工单数量", MesExtEnumUtil.ORDER_STATUS.valueOfDescription(result.getWorkOrderStatus())));
            } else {
                result.setQty(bean.getQty());
                result.setUnCompleteQty(MathOperation.sub(bean.getQty(), result.getReportedQty()));
            }
        }
        //在加工的工单不能修改物料信息
        if (result.getWorkOrderStatus() >= MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()) {
            if (!Objects.equals(bean.getPartNo(), result.getPartNo())) {
                MesException.throwMesBusiException(String.format("工单状态为【%s】不允许修改物料",
                        MesExtEnumUtil.ORDER_STATUS.valueOfDescription(result.getWorkOrderStatus())));
            }
        }

        //更新状态
        if (result.getWorkOrderStatus().intValue() != bean.getWorkOrderStatus().intValue()) {
            result.setWorkOrderStatus(bean.getWorkOrderStatus());
            if (result.getWorkOrderStatus() == MesExtEnumUtil.ORDER_STATUS.CREATE.getValue()
                    && bean.getWorkOrderStatus() == MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue()) {
                //校验物料生产版本是否存在
                MesProductVersion mesProductVersion = checkMesProductVersion(result);
                //获取bom信息
                List<MesBom> bomList = mesBomService.findMesBomByPartNoAndBomVersion(result.getPartNo(), result.getOrganizeCode(), mesProductVersion.getAlternativePartList());
                //发送工单信息给WMS
                mesWorkOrderToWmsService.insertBatch(mesWorkOrderToWmsService.saveMesWorkOrderToWms(result, bomList));
            }
        }
        updateMesWorkOrder(result,MesExtEnumUtil.WORK_ORDER_LOG_TYPE.UPDATE,0);
        return result;
    }


    @Override
    protected void setPackQueryBean(MesWorkOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partNameRdd", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCreateUser(), "createUser", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustPartNo(), "custPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderType(), "workOrderType", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getCreateDateTimeStart(), "createDatetime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getCreateDateTimeEnd(), "createDatetime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getStartTimeStart(), "planStartTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getStartTimeEnd(), "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getEndTimeStart(), "planEndTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getEndTimeEnd(), "planEndTime", packBean);
        packBean.setOrderByStr(" order by createDatetime desc,workOrderSeq asc,workOrderNo asc ");
    }

    @Override
    protected void setBeanModel(List<MesWorkOrder> mesWorkOrderList) {
        //查询工作中心名称
        if (CollectionUtils.isEmpty(mesWorkOrderList)) {
            return;
        }
        MesWorkOrder mesWorkOrder = mesWorkOrderList.iterator().next();
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getInPackList(mesWorkOrderList.stream().map(MesWorkOrder::getWorkCenterCode).filter(workCenterCode -> !StringUtils.isEmpty(workCenterCode)).distinct().collect(Collectors.toList()), "workCenterCode", ddlPackBean);
        List<MesWorkCenter> mesWorkCenterList = mesWorkCenterRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesWorkCenterList)) {
            return;
        }
        Map<String, String> mesWorkCenterMap = mesWorkCenterList.stream().collect(Collectors.toMap(MesWorkCenter::getWorkCenterCode, MesWorkCenter::getWorkCenterName));
        mesWorkOrderList.forEach(t -> {
            t.setWorkCenterName(mesWorkCenterMap.get(t.getWorkCenterCode()));
        });

    }

    @Override
    protected void onInsertBean(MesWorkOrder item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkOrderType(), "工单类型不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", item.getWorkCenterCode());
        }
        item.setErpWorkCenter(mesWorkCenter.getErpWorkCenter());
        //客户物料信息
        if (StringUtil.isEmpty(item.getCustPartNo())) {
            getCustomerPart(item);
        }
        //生成工单号
        List<String> orderList=new ArrayList<>();
        if(item.getWorkOrderType() != MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()){
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_CENTER_ORDER_NO);
            genSerialNoModel.setPartNo(item.getWorkCenterCode());
            orderList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(item.getOrganizeCode()), item.getCreateUser(), item.getOrganizeCode(), 1).getResultList();
        }else {
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_ORDER_NO);
            orderList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(item.getOrganizeCode()), item.getCreateUser(), item.getOrganizeCode(), 1).getResultList();
        }
        String orderNo = orderList.get(0);
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(orderNo, "workOrderNo", orderPackBean);
        if (baseRDao.isExitByHql(orderPackBean)) {
            MesException.throwMesBusiException("单号流水码生成重复");
        }
        item.setWorkOrderNo(orderNo);
        //校验标识
        //若工单类型为排序
        List<MesWorkOrderToWms> saveMesWorkOrderToWms = new ArrayList<>();
        if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            //校验产线类型和工单类型是否匹配
            if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.SORT.getValue())) {
                MesException.throwMesBusiException("产线【%s】类型为【%s】，请选择排序产线", mesWorkCenter.getWorkCenterCode(),
                        MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
            }
            //若工单类型为排序 标识为 N E P S
            if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue().equals(item.getOrderFlag()) &&
                    !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue().equals(item.getOrderFlag()) &&
                    !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue().equals(item.getOrderFlag()) &&
                    !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
                MesException.throwMesBusiException(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】、【%s】 标识",
                        item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue()
                        , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue(),
                        MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue(),
                        MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()));
            }
            //校验VIN号是否重复
            if (!StringUtil.isEmpty(item.getVinCode())) {
                //校验vin号是否重复
                ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getVinCode(), "vinCode", ddlPackBean);
                if (baseRDao.isExitByHql(ddlPackBean)) {
                    MesException.throwMesBusiException("vin号【%s】重复", item.getVinCode());
                }
            }
            //排序工单数量为1
            item.setQty(1d);
            //发送工单给WMS
            sendToWms(item, mesWorkCenter, saveMesWorkOrderToWms);
        } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            if (Objects.isNull(item.getQty()) || item.getQty() <= 0d) {
                MesException.throwMesBusiException("生产数量不能小于等于0");
            }
            //校验产线类型和工单类型是否匹配
            if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue())) {
                MesException.throwMesBusiException("产线【%s】类型为【%s】，请选择非排序产线", mesWorkCenter.getWorkCenterCode(),
                        MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
            }
            if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue().equals(item.getOrderFlag()) &&
                    !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue().equals(item.getOrderFlag()) &&
                    !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
                MesException.throwMesBusiException(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】 标识",
                        item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue()
                        , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue(),
                        MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()));
            }
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
            //非试制单
            if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
                ValidatorBean.checkNotNull(item.getProductVersion(), "生产版本不能为空");
                //校验物料生产版本是否存在
                MesProductVersion mesProductVersion = checkMesProductVersion(item);
                //获取bom信息
                List<MesBom> bomList = mesBomService.findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());
                //发送工单信息给WMS
                saveMesWorkOrderToWms.addAll(mesWorkOrderToWmsService.saveMesWorkOrderToWms(item, bomList));
            }
        } else {
            ValidatorBean.checkNotNull(item.getPlanStartTime(), "计划开始时间不能为空");
            ValidatorBean.checkNotNull(item.getPlanEndTime(), "计划结束时间不能为空");
            if (Objects.isNull(item.getQty()) || item.getQty() <= 0d) {
                MesException.throwMesBusiException("生产数量不能小于等于0");
            }
            //发送工单给WMS
            sendToWms(item, mesWorkCenter, saveMesWorkOrderToWms);
        }
        //物料名称不存在，从物料信息中获取
        MesPartSap mesPart = iMesPartSapService.getMesPartSapByPartNo(item.getPartNo(), item.getOrganizeCode());
        item.setPartName(mesPart.getPartName());
        item.setUnit(mesPart.getUnit());
        //生产时间新增取当天
        if (StringUtil.isEmpty(item.getProduceTime())) {
            item.setProduceTime(TimeTool.getToday());
        }
        item.setUnCompleteQty(item.getQty());
        item.setFid(UUID.randomUUID().toString());
        //保存发送WMS数据
        if (CollectionUtils.isNotEmpty(saveMesWorkOrderToWms)) {
            mesWorkOrderToWmsService.insertBatch(saveMesWorkOrderToWms);
        }
    }

    private void sendToWms(MesWorkOrder item, MesWorkCenter mesWorkCenter, List<MesWorkOrderToWms> saveMesWorkOrderToWms) {
        //非试制单
        List<MesBom> bomList = null;
        if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
            //获取生产版本
            MesProductVersion mesProductVersion = getMesProductVersion(item, mesWorkCenter);
            item.setProductVersion(mesProductVersion.getProductVersion());
            //获取bom信息
            bomList = mesBomService.findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());
        }
        //是否自动发布
        isAuto(item, saveMesWorkOrderToWms, bomList);
    }

    private void isAuto(MesWorkOrder item, List<MesWorkOrderToWms> saveMesWorkOrderToWms, List<MesBom> bomList) {
        MesPartProdGroup mesPartProdGroup = null;
        if(StringUtil.isEmpty(item.getPartProdGroupCode())){
            mesPartProdGroup = getMesPartProdGroup(item);
            item.setPartProdGroupCode(mesPartProdGroup.getPartProdGroupCode());
            item.setPartProdGroupName(mesPartProdGroup.getPartProdGroupName());
        }else{
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getPartProdGroupCode(), "partProdGroupCode", ddlPackBean);
            mesPartProdGroup = mesPartProdGroupRDao.getByProperty(ddlPackBean);
            if(Objects.isNull(mesPartProdGroup)){
                MesException.throwMesBusiException("生产线组【%s】信息不存在",item.getPartProdGroupCode());
            }
        }

        //自动发布
        if (CommonEnumUtil.VALID == mesPartProdGroup.getIsAutoRelease()) {
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
            //发送工单信息给WMS
            if(!Objects.isNull(bomList)){
                saveMesWorkOrderToWms.addAll(mesWorkOrderToWmsService.saveMesWorkOrderToWms(item, bomList));
            }
        } else {
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
        }
    }

    private MesPartProdGroup getMesPartProdGroup(MesWorkOrder item) {
        MesPartProdGroupDetail mesPartProdGroupDetail = new MesPartProdGroupDetail();
        mesPartProdGroupDetail.setWorkCenterCode(item.getWorkCenterCode());
        mesPartProdGroupDetail.setPartNo(item.getPartNo());
        mesPartProdGroupDetail.setOrganizeCode(item.getOrganizeCode());
        List<MesPartProdGroup> mesPartProdGroupList = mesPartProdGroupService.findMesPartProdGroup(mesPartProdGroupDetail);
        if (CollectionUtils.isEmpty(mesPartProdGroupList)) {
            MesException.throwMesBusiException("生产线代码【%s】零件号【%s】对应的零件生产组信息没有维护",
                    item.getWorkCenterCode(), item.getPartNo());
        }
        MesPartProdGroup mesPartProdGroup = mesPartProdGroupList.iterator().next();
        return mesPartProdGroup;
    }

    private void getCustomerPart(MesWorkOrder item) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart customerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (!Objects.isNull(customerPart)) {
            item.setCustPartNo(customerPart.getCustPartNo());
        }
    }

    private MesProductVersion checkMesProductVersion(MesWorkOrder item) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(), item.getWorkCenterCode(), item.getProductVersion());
        }
        return mesProductVersion;
    }

    private MesProductVersion getMesProductVersion(MesWorkOrder item, MesWorkCenter mesWorkCenter) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        ddlPackBean.setOrderByStr(" order by createDatetime desc ");
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesException.throwMesBusiException("物料【%s】工作中心【%s】对应的生产版本信息不存在", item.getPartNo(),
                    mesWorkCenter.getErpWorkCenter());
        }
        return mesProductVersion;
    }

    private void packPlanTime(MesWorkOrder item, Map<String, List<MesShift>> mesShiftMap) {
        ValidatorBean.checkNotNull(item.getPlanDate(), "计划日期不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");
        //工单导入：计划时间不能为当天之前日期
        try {
            if (DateUtilExt.compareDateTime(DateUtil.SHORT_FORMAT, item.getPlanDate(), TimeTool.getToday()) < 0) {
                MesException.throwMesBusiException("导入工单计划日期不能小于今天");
            }
        } catch (ImppBusiException busExcep) {
            MesException.throwMesBusiException(busExcep.getErrorDetail());
        } catch (Exception e) {
            MesException.throwMesBusiException("计划日期格式异常");
        }
        //查询班次信息
        List<MesShift> mesShifts = mesShiftMap.get(item.getWorkCenterCode() + item.getShiftCode());
        MesShift mesShift = CollectionUtils.isEmpty(mesShifts) ? null : mesShifts.iterator().next();
        if (Objects.isNull(mesShift)) {
            MesException.throwMesBusiException("产线【%s】班次【%s】信息不存在", item.getWorkCenterCode(), item.getShiftCode());
        }
        item.setShiftName(mesShift.getShiftName());
        try {
            //拼接
            item.setPlanStartTime(TimeTool.parseStringFormat(item.getPlanDate() + " " + mesShift.getStartTime(),DateUtil.BASE_FORMAT1,DateUtil.BASE_FORMAT));
            item.setPlanEndTime(TimeTool.parseStringFormat(item.getPlanDate() + " " + mesShift.getEndTime(),DateUtil.BASE_FORMAT1,DateUtil.BASE_FORMAT));
            int windowStartEndTimeCompare = DateUtilExt.compareDateTime(DateUtil.BASE_FORMAT, item.getPlanStartTime(), item.getPlanEndTime());
            if (windowStartEndTimeCompare > 0) {
                item.setPlanEndTime(TimeTool.pareDateToString(DateUtil.addDays(item.getPlanEndTime(), 1)));
            }
        } catch (Exception e) {
            MesException.throwMesBusiException("时间格式异常");
        }
        item.setProduceTime(item.getPlanDate());
    }

    @Override
    public ListPager<MesProductVersion> queryMesProductVersionByPager(MesProductVersion mesProductVersion, Pager pager) {
        DdlPackBean productVersionQueryPackBean = DdlPackBean.getDdlPackBean(mesProductVersion.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProductVersion.getPartNo(), "partNo", productVersionQueryPackBean);
        DdlPreparedPack.getStringEqualPack(mesProductVersion.getWorkCenterCode(), "workCenterCode", productVersionQueryPackBean);
        pager = PagerHelper.getPager(pager, mesProductVersionRDao.findByHqlWhereCount(productVersionQueryPackBean));
        List<MesProductVersion> mesProductVersions = mesProductVersionRDao.findByHqlWherePage(productVersionQueryPackBean, pager);
        return new ListPager<>(mesProductVersions, pager);
    }

    @Override
    public void saveMesWorkOrderExcel(List<MesWorkOrder> mesWorkOrderList) {
        MesWorkOrder next = mesWorkOrderList.iterator().next();
        //属性校验&字段填充
        List<MesWorkOrderToWms> mesWorkOrderToWmsList = checkData(mesWorkOrderList, next.getOrganizeCode());
        //BTO序号
        List<String> seqList = new ArrayList<>();
        if (MesExtEnumUtil.ORDER_TYPE.BTO.getValue() == next.getWorkOrderType()) {
            GenSerialNoModel orderSeqSerialNoModel = new GenSerialNoModel(MesCommonConstant.ORDER_NO_BTO_SEQ);
            seqList = syncFuncService.syncSerialNo(orderSeqSerialNoModel.organizeCode(next.getOrganizeCode()), next.getCreateUser(), next.getOrganizeCode(), mesWorkOrderList.size()).getResultList();
        }
        List<MesWorkOrderLog> mesWorkOrderLogList = new ArrayList<>();
        int index = 0;
        for (MesWorkOrder item : mesWorkOrderList) {
            //获取相同类型 相同标识 生产序号最大值的
            DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(item.getWorkOrderStatus(), "workOrderStatus", orderPackBean);
            DdlPreparedPack.getStringEqualPack(item.getOrderFlag(), "orderFlag", orderPackBean);
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"produceSeq"}, orderPackBean);
            MesWorkOrder result = baseRDao.getByProperty(orderPackBean);

            //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
                Long produceSeq = 0L;
                if (null != result && result.getProduceSeq() != null) {
                    produceSeq = (long) Math.toIntExact(result.getProduceSeq());
                }
                MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
                BeanUtils.copyProperties(item, copyMesWorkOrder);
                copyMesWorkOrder.setProduceSeq((long) (produceSeq + 1));
                copyMesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                ConvertBean.serviceModelInitialize(copyMesWorkOrder, item.getCreateUser());
                index++;
                baseRDao.insert(copyMesWorkOrder);
                //保存记录
                MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
                BeanUtils.copyProperties(item, workOrderLog, "id");
                mesWorkOrderLogList.add(workOrderLog);
                continue;
            } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.BTO.getValue()) {
                Long produceSeq = 0L;
                if (null != result && result.getProduceSeq() != null) {
                    produceSeq = (long) Math.toIntExact(result.getProduceSeq());
                }
                item.setProduceSeq(produceSeq + 1);
                item.setWorkOrderSeq(seqList.get(index));
            }
            item.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            ConvertBean.serviceModelInitialize(item, item.getCreateUser());
            baseRDao.insert(item);
            index++;
            //保存记录
            MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
            BeanUtils.copyProperties(item, workOrderLog, "id");
            mesWorkOrderLogList.add(workOrderLog);
        }
        //保存发送WMS数据
        if (CollectionUtils.isNotEmpty(mesWorkOrderToWmsList)) {
            mesWorkOrderToWmsService.insertBatch(mesWorkOrderToWmsList);
        }
        //保存工单记录
        if (CollectionUtils.isNotEmpty(mesWorkOrderLogList)) {
            workOrderLogService.insertBatch(mesWorkOrderLogList);
        }
    }

    private List<MesWorkOrderToWms> checkData(List<MesWorkOrder> mesWorkOrderList, String organizeCode) {
        MesWorkOrder next = mesWorkOrderList.iterator().next();
        String nowDate = TimeTool.getNowTime(true);
        //获取产线信息
        List<String> workCenterCodeList = mesWorkOrderList.stream().map(MesWorkOrder::getWorkCenterCode).distinct().collect(Collectors.toList());
        List<MesWorkCenter> mesWorkCenterList = BeanMapUtilsExt.getBeanList(mesWorkCenterRDao, "workCenterCode", organizeCode, workCenterCodeList, "产线信息");
        //产线导入不区分大小小
        Map<String, MesWorkCenter> workCenterMap = new CaseInsensitiveMap();
        for (MesWorkCenter workCenter : mesWorkCenterList) {
            workCenterMap.put(workCenter.getWorkCenterCode(), workCenter);
        }
        //获取班次信息
        List<MesShift> mesShiftList = getMesShifts(organizeCode, nowDate, workCenterCodeList);
        Map<String, List<MesShift>> mesShiftMap = mesShiftList.stream().collect(Collectors.groupingBy(t -> t.getWorkCenterCode() + t.getShiftCode()));
        //获取生产版本
        List<String> partNoList = mesWorkOrderList.stream().map(MesWorkOrder::getPartNo).distinct().collect(Collectors.toList());
        List<MesProductVersion> mesProductVersionList = BeanMapUtilsExt.getBeanList(mesProductVersionRDao, "partNo", organizeCode, partNoList, "生产版本信息");
        Map<String, List<MesProductVersion>> mesProductVersionMap = mesProductVersionList.stream().collect(Collectors.groupingBy(t -> t.getWorkCenterCode() + t.getPartNo()));
        //客户物料信息
        Map<String, MesCustomerPart> mesCustomerPartMap = null;
        try {
            mesCustomerPartMap = BeanMapUtilsExt.getBeanMap(mesCustomerPartRDao, "erpPartNo", organizeCode, partNoList, "客户物料信息");
        } catch (Exception e) {
            mesCustomerPartMap = null;
        }
        //获取物料信息
        Map<String, MesPartSap> mesPartSapMap = BeanMapUtilsExt.getBeanMap(mesPartSapRDao, "partNo", organizeCode, partNoList, "物料信息");
        //获取获取vin号
        List<String> vinCodeList = mesWorkOrderList.stream().map(MesWorkOrder::getVinCode).filter(Objects::nonNull).distinct().collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(vinCodeList)) {
            List<MesWorkOrder> mesWorkOrders = null;
            try {
                mesWorkOrders = BeanMapUtilsExt.getBeanList(baseRDao, "vinCode", organizeCode, vinCodeList, "生产工单");
            } catch (Exception e) {
            }
            if (!CollectionUtils.isEmpty(mesWorkOrders)) {
                List<String> list = mesWorkOrders.stream().map(MesWorkOrder::getVinCode).collect(Collectors.toList());
                MesException.throwMesBusiException("vin号【%s】已存在", list);
            }
        }
        //生成工单号
        List<String> orderList=new ArrayList<>();
        if(next.getWorkOrderType() != MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()){
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_CENTER_ORDER_NO);
            genSerialNoModel.setPartNo(next.getWorkCenterCode());
            orderList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(next.getOrganizeCode()), next.getCreateUser(), next.getOrganizeCode(), mesWorkOrderList.size()).getResultList();
        }else {
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_ORDER_NO);
            orderList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(next.getOrganizeCode()), next.getCreateUser(), next.getOrganizeCode(), mesWorkOrderList.size()).getResultList();
        }
        //查询工单是否已经存在
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(next.getOrganizeCode());
        DdlPreparedPack.getInPackList(orderList, "workOrderNo", orderPackBean);
        if (baseRDao.isExitByHql(orderPackBean)) {
            MesException.throwMesBusiException("单号已存在");
        }

        //BOM信息
        Map<String, List<MesBom>> mesBomMap = new HashMap<>();
        //vin号
        Map<String, String> vinCodeMap = new HashMap<>();
        //零件生产组
        Map<String, MesPartProdGroup> mesPartProdGroupMap = new HashMap<>();
        //发送wms数据
        List<MesWorkOrderToWms> mesWorkOrderToWmsList = new ArrayList<>();
        int index = 0;
        for (MesWorkOrder item : mesWorkOrderList) {
            item.setWorkOrderNo(orderList.get(index));
            // 数据校验
            ValidatorBean.checkNotNull(item.getWorkOrderType(), "工单类型不能为空");
            ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");
            ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");

            MesWorkCenter mesWorkCenter = workCenterMap.get(item.getWorkCenterCode());

            if (Objects.isNull(mesWorkCenter)) {
                MesException.throwMesBusiException("产线【%s】不存在", item.getWorkCenterCode());
            }
            item.setWorkCenterCode(mesWorkCenter.getWorkCenterCode());
            item.setErpWorkCenter(mesWorkCenter.getErpWorkCenter());
            //获取客户物料信息
            MesCustomerPart customerPart = null == mesCustomerPartMap ? null : mesCustomerPartMap.get(item.getPartNo());
            if (!Objects.isNull(customerPart)) {
                item.setCustPartNo(customerPart.getCustPartNo());
                item.setCustomerPartName(customerPart.getCustPartName());
            }
            //物料名称不存在，从物料信息中获取
            MesPartSap mesPart = mesPartSapMap.get(item.getPartNo());
            if (Objects.isNull(mesPart)) {
                MesException.throwMesBusiException("物料【%s】信息不存在", item.getPartNo());
            }
            item.setPartName(mesPart.getPartName());
            item.setUnit(mesPart.getUnit());
            //生产时间新增取当天
            if (StringUtil.isEmpty(item.getProduceTime())) {
                item.setProduceTime(TimeTool.getToday());
            }
            //若工单类型为排序
            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
                ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");
                ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
                //校验产线类型和工单类型是否匹配
                if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.SORT.getValue())) {
                    MesException.throwMesBusiException("产线【%s】类型为【%s】，请选择排序产线", mesWorkCenter.getWorkCenterCode(),
                            MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
                }
                //若工单类型为排序 标识为 N E P S
                if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue().equals(item.getOrderFlag()) &&
                        !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue().equals(item.getOrderFlag()) &&
                        !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue().equals(item.getOrderFlag()) &&
                        !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
                    MesException.throwMesBusiException(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】、【%s】 标识",
                            item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue()
                            , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue(),
                            MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue(),
                            MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()));
                }
                //计划开始时间，计划结束时间拼接
                packPlanTime(item, mesShiftMap);
                //校验vin号
                if (!StringUtil.isEmpty(item.getVinCode())) {
                    if (vinCodeMap.containsKey(item.getVinCode())) {
                        MesException.throwMesBusiException("vin号【%s】重复", item.getVinCode());
                    } else {
                        vinCodeMap.put(item.getVinCode(), item.getVinCode());
                    }
                }
                //排序工单数量为1
                item.setQty(1d);
                //发送wms数据
                sendToWms(mesProductVersionMap, mesBomMap, mesPartProdGroupMap, mesWorkOrderToWmsList, item, mesWorkCenter);
            } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
                ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");
                ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
                if (Objects.isNull(item.getQty()) || item.getQty() <= 0d) {
                    MesException.throwMesBusiException("生产数量不能小于等于0");
                }
                //校验产线类型和工单类型是否匹配
                if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue())) {
                    MesException.throwMesBusiException("产线【%s】类型为【%s】，请选择非排序产线", mesWorkCenter.getWorkCenterCode(),
                            MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
                }
                if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue().equals(item.getOrderFlag()) &&
                        !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue().equals(item.getOrderFlag()) &&
                        !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())) {
                    MesException.throwMesBusiException(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】 标识",
                            item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue()
                            , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue(),
                            MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()));
                }
                //拼接时间
                packPlanTime(item, mesShiftMap);
                //状态已发布
                item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
                List<MesBom> mesBomList = null;
                if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())){
                    ValidatorBean.checkNotNull(item.getProductVersion(), "生产版本不能为空");
                    //校验物料生产版本是否存在
                    List<MesProductVersion> mesProductVersions = mesProductVersionMap.get(mesWorkCenter.getErpWorkCenter() + item.getPartNo());
                    if (CollectionUtils.isEmpty(mesProductVersions)) {
                        MesException.throwMesBusiException("物料【%s】工作中心【%s】对应的生产版本信息不存在", item.getPartNo(),
                                mesWorkCenter.getErpWorkCenter());
                    }
                    List<MesProductVersion> versionList = mesProductVersions.stream().filter(t -> Objects.equals(item.getProductVersion(), t.getProductVersion())).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(versionList)) {
                        MesException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(), mesWorkCenter.getWorkCenterCode(), item.getProductVersion());
                    }
                    MesProductVersion mesProductVersion = versionList.iterator().next();
                    //获取bom信息
                    if (mesBomMap.containsKey(item.getPartNo() + mesProductVersion.getAlternativePartList())) {
                        mesBomList = mesBomMap.get(item.getPartNo() + mesProductVersion.getAlternativePartList());
                    } else {
                        //获取bom信息
                        mesBomList = mesBomService.findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());
                        mesBomMap.put(item.getPartNo() + mesProductVersion.getAlternativePartList(), mesBomList);
                    }
                    //发送工单信息给WMS
                    mesWorkOrderToWmsList.addAll(mesWorkOrderToWmsService.saveMesWorkOrderToWms(item, mesBomList));
                }
            } else {
                ValidatorBean.checkNotNull(item.getPlanStartTime(), "计划开始时间不能为空");
                ValidatorBean.checkNotNull(item.getPlanEndTime(), "计划结束时间不能为空");
                if (Objects.isNull(item.getQty()) || item.getQty() <= 0d) {
                    MesException.throwMesBusiException("生产数量不能小于等于0");
                }
                //发送wms数据
                sendToWms(mesProductVersionMap, mesBomMap, mesPartProdGroupMap, mesWorkOrderToWmsList, item, mesWorkCenter);
            }
            item.setUnCompleteQty(item.getQty());
            item.setFid(UUID.randomUUID().toString());
            index++;
        }
        return mesWorkOrderToWmsList;
    }

    private void sendToWms(Map<String, List<MesProductVersion>> mesProductVersionMap, Map<String, List<MesBom>> mesBomMap, Map<String, MesPartProdGroup> mesPartProdGroupMap, List<MesWorkOrderToWms> mesWorkOrderToWmsList, MesWorkOrder item, MesWorkCenter mesWorkCenter) {
        //发送工单信息给WMS
        List<MesBom> mesBomList = null;
        if (!MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(item.getOrderFlag())){
            //获取生产版本
            List<MesProductVersion> mesProductVersions = mesProductVersionMap.get(mesWorkCenter.getErpWorkCenter() + item.getPartNo());
            if (CollectionUtils.isEmpty(mesProductVersions)) {
                MesException.throwMesBusiException("物料【%s】工作中心【%s】对应的生产版本信息不存在", item.getPartNo(),
                        mesWorkCenter.getErpWorkCenter());
            }
            MesProductVersion mesProductVersion = mesProductVersions.iterator().next();
            item.setProductVersion(mesProductVersion.getProductVersion());
            if (mesBomMap.containsKey(item.getPartNo() + mesProductVersion.getAlternativePartList())) {
                mesBomList = mesBomMap.get(item.getPartNo() + mesProductVersion.getAlternativePartList());
            } else {
                //获取bom信息
                mesBomList = mesBomService.findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());
                mesBomMap.put(item.getPartNo() + mesProductVersion.getAlternativePartList(), mesBomList);
            }
        }
        //是否自动发布
        isAuto(mesPartProdGroupMap, mesWorkOrderToWmsList, item, mesBomList);
    }

    private void isAuto(Map<String, MesPartProdGroup> mesPartProdGroupMap, List<MesWorkOrderToWms> mesWorkOrderToWmsList, MesWorkOrder item, List<MesBom> mesBomList) {
        //校验零件生产组信息
        MesPartProdGroup mesPartProdGroup;
        if (mesPartProdGroupMap.containsKey(item.getWorkCenterCode() + item.getPartNo())) {
            mesPartProdGroup = mesPartProdGroupMap.get(item.getWorkCenterCode() + item.getPartNo());
        } else {
            mesPartProdGroup = getMesPartProdGroup(item);
            mesPartProdGroupMap.put(item.getWorkCenterCode() + item.getPartNo(), mesPartProdGroup);
        }
        item.setPartProdGroupCode(mesPartProdGroup.getPartProdGroupCode());
        item.setPartProdGroupName(mesPartProdGroup.getPartProdGroupName());
        //自动发布
        if (CommonEnumUtil.VALID == mesPartProdGroup.getIsAutoRelease()) {
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
            //发送工单信息给WMS
            if(!Objects.isNull(mesBomList)){
                mesWorkOrderToWmsList.addAll(mesWorkOrderToWmsService.saveMesWorkOrderToWms(item, mesBomList));
            }
        } else {
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
        }
    }

    private List<MesShift> getMesShifts(String organizeCode, String nowDate, List<String> workCenterCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(workCenterCodeList, "workCenterCode", packBean);
        DdlPreparedPack.getStringSmallerPack(nowDate, "beginDate", packBean);
        DdlPreparedPack.getStringBiggerPack(nowDate, "endDate", packBean);
        List<MesShift> mesShiftList = mesShiftRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesShiftList)) {
            MesException.throwMesBusiException("有效期内的班次信息不存在");
        }
        return mesShiftList;
    }

    @Override
    public void doProductReport(MesWorkOrder mesWorkOrder) {
        MesWorkOrder mesWorkOrderDb = baseRDao.getById(mesWorkOrder.getId());
        if (Objects.isNull(mesWorkOrderDb)) {
            MesException.throwMesBusiException("工单id为【%s】工单信息不存在", mesWorkOrder.getId());
        }
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(mesWorkOrderDb.getPartNo(), mesWorkOrderDb.getOrganizeCode());
        //查询产线信息
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", mesWorkOrderDb.getWorkCenterCode());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrderDb);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms = mesBomService.findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //报工类型
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrder.getReportType()) {
            mesWorkOrderDb.setReportedQty(MathOperation.add(mesWorkOrder.getNum(), mesWorkOrderDb.getReportedQty()));
            //更新工单状态
            double unCompleteQty = MathOperation.sub(mesWorkOrderDb.getQty(), mesWorkOrderDb.getReportedQty());
            mesWorkOrderDb.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (mesWorkOrderDb.getReportedQty() > mesWorkOrderDb.getQty()) {
                //查询产线信息
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", packBean);
                MesWorkCenter workCenter = mesWorkCenterRDao.getByProperty(packBean);
                // 以下则是超工单逻辑
                // 如果产线中没有配置超工单，则直接阻断
                if (!Objects.equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), workCenter.getIsCheckOrderQty())) {
                    MesException.throwMesBusiException("请检查工单数量,工单号[%s],工单数量[%s]报工数量[%s],且未配置超工单!", mesWorkOrder.getWorkOrderNo(), mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
                }
                // 如果配置了超工单，且比例已经超过了配置的超工单比例，也需要阻断
                double rate =  MathOperation.div((mesWorkOrderDb.getReportedQty() - mesWorkOrderDb.getQty()),mesWorkOrderDb.getQty());
                if (rate > MathOperation.div(workCenter.getOrderRate(),100)) {
                    MesException.throwMesBusiException("请检查工单数量,工单号[%s],工单数量[%s]报工数量[%s],配置了超工单,但超过了比例[%s]!", mesWorkOrder.getWorkOrderNo(), mesWorkOrder.getQty(), mesWorkOrder.getReportedQty(), workCenter.getOrderRate());
                }
                mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else if (Objects.equals(mesWorkOrderDb.getReportedQty(), mesWorkOrderDb.getQty())) {
                mesWorkOrderDb.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                mesWorkOrderDb.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
            mesWorkOrderDb.setCompleteQty(mesWorkOrderDb.getReportedQty());
        } else {
            //报工调整数量加-
            mesWorkOrderDb.setAdjustQty((MathOperation.add(mesWorkOrder.getNum(), mesWorkOrderDb.getAdjustQty())));
            //冲销数量大于工单完成数量报错
            if (mesWorkOrderDb.getAdjustQty() > mesWorkOrderDb.getReportedQty()) {
                MesException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrderDb.getAdjustQty(), mesWorkOrderDb.getReportedQty());
            }
        }
        //更新工单
        updateMesWorkOrder(mesWorkOrderDb,(MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrder.getReportType()) ? MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT : MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT_ADJUST,mesWorkOrder.getNum());
        //试制单不报工
        if(Objects.isNull(mesWorkOrderDb.getOrderFlag()) || !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(mesWorkOrderDb.getOrderFlag())){
            String sn = getStringList(mesWorkOrder);
            //保存数据
            List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
            for (MesBom mesBom : mesBoms) {
                mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
            }
            mesProductOffLineService.insertBatch(mesProductOffLineList);
        }
    }

    @Override
    public void doProductReportExt(MesWorkOrder mesWorkOrderNew) {
        MesWorkOrder mesWorkOrder = baseRDao.getById(mesWorkOrderNew.getId());
        if (Objects.isNull(mesWorkOrder)) {
            MesException.throwMesBusiException("工单id为【%s】工单信息不存在", mesWorkOrderNew.getId());
        }
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
        if (mesWorkOrderNew.getNum() <= 0) {
            MesException.throwMesBusiException("工单报工数量不能为0,不允许报工");
        }
        //报工类型
        //报工
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrderNew.getReportType()) {
            mesWorkOrder.setReportedQty(MathOperation.add(mesWorkOrderNew.getNum(), mesWorkOrder.getReportedQty()));
            //更新工单状态
            double unCompleteQty = MathOperation.sub(mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
            mesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (mesWorkOrder.getReportedQty() > mesWorkOrder.getQty()) {
                //查询产线信息
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
                MesWorkCenter workCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
                // 以下则是超工单逻辑
                // 如果产线中没有配置超工单，则直接阻断
                if (!Objects.equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), workCenter.getIsCheckOrderQty())) {
                    MesException.throwMesBusiException("请检查工单数量,工单号[%s],工单数量[%s]报工数量[%s],且未配置超工单!", mesWorkOrder.getWorkOrderNo(), mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
                }
                // 如果配置了超工单，且比例已经超过了配置的超工单比例，也需要阻断
                double rate = (mesWorkOrder.getReportedQty() - mesWorkOrder.getQty())/mesWorkOrder.getQty();
                if (rate > MathOperation.div(workCenter.getOrderRate(),100)) {
                    MesException.throwMesBusiException("请检查工单数量,工单号[%s],工单数量[%s]报工数量[%s],配置了超工单,但超过了比例[%s]!", mesWorkOrder.getWorkOrderNo(), mesWorkOrder.getQty(), mesWorkOrder.getReportedQty(), workCenter.getOrderRate());
                }
                mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else if (Objects.equals(mesWorkOrder.getReportedQty(), mesWorkOrder.getQty())) {
                mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
            mesWorkOrder.setCompleteQty(mesWorkOrder.getReportedQty());
        } else {
            //报工调整数量加-
            mesWorkOrder.setAdjustQty((MathOperation.add(mesWorkOrderNew.getNum(), mesWorkOrder.getAdjustQty())));
            //冲销数量大于工单完成数量报错
            if (mesWorkOrder.getAdjustQty() > mesWorkOrder.getReportedQty()) {
                MesException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrder.getAdjustQty(), mesWorkOrder.getReportedQty());
            }
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrder);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms = mesBomService.findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //生成条码
        String sn = getStringList(mesWorkOrderNew);
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        List<MesMove> mesMoveList = new ArrayList<>();
        boolean isReport = false;
        boolean isMove = false;
        //排序工单
        if (MesExtEnumUtil.ORDER_TYPE.SORT.getValue() == mesWorkOrder.getWorkOrderType()) {
            //报工类型
            if (MesExtEnumUtil.REPORT_TYPE.WRITE_OFF.getValue() == mesWorkOrder.getReportType()) {
                MesException.throwMesBusiException("工单【%s】为排序工单不允许冲销", mesWorkOrder.getWorkOrderNo());
            }
            //排序加工规则
            MesProdRuleSortCfg mesProdRuleSortCfg = getMesProdRuleSortCfg(mesWorkOrder);
            if (MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue() == mesProdRuleSortCfg.getReportType()) {
                isReport = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                isMove = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.REPORT_MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                isReport = true;
                isMove = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.CUSTOMER_SUPPLY_MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                customerSupplyMove(mesWorkOrderNew, mesProductVersion, mesBoms, sn, mesProductOffLineList, mesMoveList);
            } else {
                log.info("工单{}排序加工规则报工类型未维护", mesWorkOrder.getWorkOrderNo());
                return;
            }

            if (MesExtEnumUtil.MES_REPORT_TYPE.CUSTOMER_SUPPLY_MOVE.getValue() != mesProdRuleSortCfg.getReportType()) {
                //查询SAP物料信息
                MesPartSap mesPartSap = mesPartSapService.getMesPartSapByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
                //成品汇报
                if (isReport) {
                    for (MesBom mesBom : mesBoms) {
                        mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrderNew, mesProductVersion, sn, mesBom, false));
                    }
                }
                //成品移库
                if (isMove) {
                    MesMove move = createMove(mesPartSap, mesProdRuleSortCfg.getSrcErpLocation(), mesProdRuleSortCfg.getDestErpLocation(), mesWorkOrder.getOrganizeCode(), mesWorkOrder.getWorkCenterCode(), mesWorkOrderNew.getNum(), sn, MesExtEnumUtil.MOVE_TYPE.FINISH_PRODUCTS_MOVE.getValue());
                    move.setMatnr(mesWorkOrder.getPartNo());
                    mesMoveList.add(move);
                }
            }
        } else if (MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue() == mesWorkOrder.getWorkOrderType()) {
            MesProdRuleNosortCfg mesProdRuleNoSortCfg = mesProdRuleNosortCfgService.findMesProdRuleNosortCfgByWorkCenterCode(mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
            if (MesExtEnumUtil.NOSORT_REPORT_TYPE.REPORT.getValue() == mesProdRuleNoSortCfg.getReportType()) {
                for (MesBom mesBom : mesBoms) {
                    mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrderNew, mesProductVersion, sn, mesBom, false));
                }

            }
        } else {
            log.info("工单{}排序规则不属于排序工单和非排序工单", mesWorkOrder.getWorkOrderNo());
            return;
        }

        //更新工单
        updateMesWorkOrder(mesWorkOrder, (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrderNew.getReportType()) ? MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT : MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT_ADJUST,mesWorkOrderNew.getNum());

        //保存报工记录 试制单不报工
        if (CollectionUtils.isNotEmpty(mesProductOffLineList) && (Objects.isNull(mesWorkOrder.getOrderFlag()) || !MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue().equals(mesWorkOrder.getOrderFlag()))) {
            mesProductOffLineService.insertBatch(mesProductOffLineList);
        }
        //保存移库记录
        if (CollectionUtils.isNotEmpty(mesMoveList)) {
            mesMoveRDao.saveAll(mesMoveList);
        }
    }

    @Override
    public void deleteUnSortOrder(String organizeCode, List<String> partNoList) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue(), "workOrderType", ddlPackBean);
        DdlPreparedPack.getStringSmallerNotEqualPack(TimeTool.getToday(), "planStartTime", ddlPackBean);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue(), "workOrderStatus", ddlPackBean);
        baseRDao.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "workOrderStatus"},
                new Object[]{"CLOSE_JOB", TimeTool.getNowTime(true), MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue()}, ddlPackBean);
    }


    @Override
    public void doReportWorkReversal(WmsProductModel model) {
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(model.getPartNo(), model.getOrganizeCode());
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(model);
        //获取bom
        List<MesBom> mesBoms = mesBomService.findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        for (MesBom mesBom : mesBoms) {
            mesProductOffLineList.add(getMesProductOffLine(model, mesPart, mesProductVersion, mesBom));
        }
        mesProductOffLineService.insertBatch(mesProductOffLineList);
    }

    @Override
    public void doReopen(String[] ids,String userName,String organizeCode) {
        //获取工单信息
        List<MesWorkOrder> mesWorkOrderList = getMesWorkOrderList(ids, organizeCode);
        //数据更新
        for (MesWorkOrder mesWorkOrder : mesWorkOrderList) {
            if (MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue() != mesWorkOrder.getWorkOrderStatus()) {
                MesException.throwMesBusiException("工单【%S】状态不为关闭，请检查数据", mesWorkOrder.getWorkOrderNo());
            }
            if (!MesExtEnumUtil.OK_NOK.NOK.getValue().equals(mesWorkOrder.getCcscStatus())) {
                MesException.throwMesBusiException("工单【%S】CCSC状态不为NOK，请检查数据", mesWorkOrder.getWorkOrderNo());
            }
            if (mesWorkOrder.getUnCompleteQty() <= 0) {
                MesException.throwMesBusiException("工单【%s】未完成数量小于等于0，请检查数据", mesWorkOrder.getWorkOrderNo());
            }
            //更新工单状态
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            mesWorkOrder.setCcscStatus("");
            updateMesWorkOrder(mesWorkOrder,MesExtEnumUtil.WORK_ORDER_LOG_TYPE.UPDATE,mesWorkOrder.getQty());
        }
    }

    private List<MesWorkOrder> getMesWorkOrderList(String[] ids, String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPack(String.join(",", ids),"id",packBean);
        List<MesWorkOrder> mesWorkOrderList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesWorkOrderList)) {
            MesException.throwMesBusiException("选择工单信息不存在");
        }
        return mesWorkOrderList;
    }

    private void updateMesWorkOrder(MesWorkOrder mesWorkOrder,MesExtEnumUtil.WORK_ORDER_LOG_TYPE type,double qty) {
        //修改工单，需要重新同步
        mesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        ConvertBean.serviceModelUpdate(mesWorkOrder, AuthUtil.getSessionUser().getUserName());
        baseRDao.update(mesWorkOrder);
        //保存记录
        workOrderLogService.saveMesWorkOrderLog(mesWorkOrder,type,qty);
    }

    private MesProductOffLine getMesProductOffLine(WmsProductModel model, MesPart mesPart, MesProductVersion mesProductVersion, MesBom mesBom) {
        MesProductOffLine newMesProductOffLine = new MesProductOffLine();
        newMesProductOffLine.setReportPartNo(mesPart.getPartNo());
        newMesProductOffLine.setReportPartNameRdd(mesPart.getPartName());
        newMesProductOffLine.setReportSn(model.getSn());
        newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
        newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
        newMesProductOffLine.setItemQty(MathOperation.mul(model.getQty(), mesBom.getItemQty()));
        newMesProductOffLine.setWorkCenterCode(mesProductVersion.getWorkCenterCode());
        newMesProductOffLine.setSapWorkCenter(mesProductVersion.getWorkCenterCode());
        newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
        newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
        newMesProductOffLine.setQty(model.getQty());
        newMesProductOffLine.setBomVersion(mesProductVersion.getProductVersion());
        newMesProductOffLine.setSerialNumber(model.getSn());
        newMesProductOffLine.setUnit(mesBom.getUnit());
        newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
        newMesProductOffLine.setReportType(MesExtEnumUtil.REPORT_TYPE.CBR.getValue());
        newMesProductOffLine.setOrganizeCode(model.getOrganizeCode());
        newMesProductOffLine.setOrderNo(model.getShipOrderNo());
        newMesProductOffLine.setCustNo(model.getCustNo());
        newMesProductOffLine.setCustOrder(model.getCustOrderNo());

        ConvertBean.serviceModelInitialize(newMesProductOffLine, model.getUserName());
        return newMesProductOffLine;
    }

    private MesProductVersion getMesProductVersion(WmsProductModel model) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(model.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getOrderBy(null, null, ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesException.throwMesBusiException("物料【%s】生产版本信息不存在", model.getPartNo());
        }
        return mesProductVersion;
    }

    private void customerSupplyMove(MesWorkOrder mesWorkOrder, MesProductVersion mesProductVersion, List<MesBom> mesBoms, String sn, List<MesProductOffLine> mesProductOffLineList, List<MesMove> mesMoveList) {
        boolean isItemMove;
        boolean isItemReport;
        MesMoveRule moveRule;
        //查询零件生产组的移库规则
        List<MesMoveRule> moveRules = mesMoveRuleService.findMesMoveRuleByPartProdGroupCode(mesWorkOrder.getPartProdGroupCode(), mesWorkOrder.getOrganizeCode());
        Map<String, List<MesMoveRule>> mesMoveRuleMap = moveRules.stream().filter(t -> Objects.nonNull(t.getSrcType())).collect(Collectors.groupingBy(MesMoveRule::getSrcType));
        List<String> itemPartNoList = mesBoms.stream().map(MesBom::getItemPartNo).collect(Collectors.toList());
        //子物料SAP下发信息
        Map<String, List<MesPartSap>> mesPartSapMap = getPartSapMap(mesWorkOrder, itemPartNoList);
        //获取物料信息
        Map<String, List<MesPart>> itemPartMap = getItemPartMap(mesWorkOrder, itemPartNoList);
        for (MesBom mesBom : mesBoms) {
            isItemReport = false;
            isItemMove = false;
            moveRule = null;
            if (itemPartMap.containsKey(mesBom.getItemPartNo())) {
                MesPart itemPart = itemPartMap.get(mesBom.getItemPartNo()).iterator().next();
                if (!StringUtil.isEmpty(itemPart.getEsd()) && mesMoveRuleMap.containsKey(itemPart.getEsd())) {
                    moveRule = mesMoveRuleMap.get(itemPart.getEsd()).iterator().next();
                    if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.REPORT_MOVE.getValue() == moveRule.getReportType()) {
                        isItemMove = true;
                        isItemReport = true;
                    } else if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.REPORT.getValue() == moveRule.getReportType()) {
                        isItemReport = true;
                    } else if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.MOVE.getValue() == moveRule.getReportType()) {
                        isItemMove = true;
                    }
                }
            }
            //汇报
            if (isItemReport) {
                mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, true));
            }
            //移库
            if (isItemMove) {
                MesPartSap mesPartSap = null;
                if (mesPartSapMap.containsKey(mesBom.getItemPartNo())) {
                    mesPartSap = mesPartSapMap.get(mesBom.getItemPartNo()).iterator().next();
                }
                MesMove move = createMove(mesPartSap, moveRule.getErpSrcLocateNo(), moveRule.getErpDestLocateNo(), mesBom.getOrganizeCode(), mesWorkOrder.getWorkCenterCode(), mesBom.getItemQty(), "", MesExtEnumUtil.MOVE_TYPE.RAW_MATERIAL_MOVE.getValue());
                move.setMatnr(mesBom.getItemPartNo());
                mesMoveList.add(move);
            }
        }
    }

    private Map<String, List<MesPartSap>> getPartSapMap(MesWorkOrder mesWorkOrder, List<String> itemPartNoList) {
        List<MesPartSap> mesPartSapList = mesPartSapService.findMesPartSapByPartNoList(itemPartNoList, mesWorkOrder.getOrganizeCode());
        Map<String, List<MesPartSap>> mesPartSapMap = mesPartSapList.stream().filter(t -> Objects.nonNull(t.getPartNo())).collect(Collectors.groupingBy(MesPartSap::getPartNo));
        return mesPartSapMap;
    }

    private Map<String, List<MesPart>> getItemPartMap(MesWorkOrder mesWorkOrder, List<String> itemPartNoList) {
        List<MesPart> itemPartList = iMesPartService.findMesPartByPartNoList(itemPartNoList, mesWorkOrder.getOrganizeCode());
        Map<String, List<MesPart>> itemPartMap = itemPartList.stream().filter(t -> Objects.nonNull(t.getPartNo())).collect(Collectors.groupingBy(MesPart::getPartNo));
        return itemPartMap;
    }

    private MesProdRuleSortCfg getMesProdRuleSortCfg(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "reportPartNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesProdRuleSortCfg mesProdRuleSortCfg = mesProdRuleSortCfgRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProdRuleSortCfg)) {
            MesException.throwMesBusiException("产线【%s】物料【%s】排序加工规则未维护", mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getPartNo());
        }
        return mesProdRuleSortCfg;
    }

    private String getStringList(MesWorkOrder mesWorkOrder) {
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.REPORT_SN);
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(mesWorkOrder.getOrganizeCode()), mesWorkOrder.getCreateUser(), mesWorkOrder.getOrganizeCode(), 1).getResultList();
        return resultList.iterator().next();
    }


    private MesProductVersion getMesProductVersion(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesException.throwMesBusiException("物料【%s】生产版本信息不存在", mesWorkOrder.getPartNo());
        }
        return mesProductVersion;
    }


    private MesProductOffLine creatMesProductOffLine(MesWorkOrder mesWorkOrder, MesProductVersion mesProductVersion,
                                                     String sn, MesBom mesBom, boolean isItemReport) {
        MesProductOffLine newMesProductOffLine;
        newMesProductOffLine = new MesProductOffLine();
        if (!isItemReport) {
            newMesProductOffLine.setReportPartNo(mesWorkOrder.getPartNo());
            newMesProductOffLine.setReportPartNameRdd(mesWorkOrder.getPartName());
        }

        newMesProductOffLine.setReportSn(sn);
        newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
        newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
        newMesProductOffLine.setItemQty(MathOperation.mul(mesWorkOrder.getNum(), mesBom.getItemQty()));
        newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
        newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
        newMesProductOffLine.setQty(Double.valueOf(mesWorkOrder.getNum()));
        newMesProductOffLine.setBomVersion(mesWorkOrder.getProductVersion());
        newMesProductOffLine.setSerialNumber(sn);
        newMesProductOffLine.setUnit(mesBom.getUnit());
        newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
        newMesProductOffLine.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
        newMesProductOffLine.setWorkCenterCode(mesWorkOrder.getWorkCenterCode());
        newMesProductOffLine.setWorkCellCode(mesWorkOrder.getWorkCellCode());
        newMesProductOffLine.setReportType(mesWorkOrder.getReportType());
        newMesProductOffLine.setSapWorkCenter(mesWorkOrder.getErpWorkCenter());
        newMesProductOffLine.setOrganizeCode(mesWorkOrder.getOrganizeCode());

        ConvertBean.serviceModelInitialize(newMesProductOffLine, mesWorkOrder.getCreateUser());
        return newMesProductOffLine;
    }

    @Override
    public List<MesWorkOrderModel> sumMesWorkOrderQtyToSap(String organizeCode) {
        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
//        statusList.add(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue());
        //查询汇总工单数量
        return mesWorkOrderDao.sumMesWorkOrderQtyToSap(organizeCode, statusList);
    }

    @Override
    public List<MesWorkOrder> queryMesWorkOrderProcessToSap(String organizeCode) {
        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
//        statusList.add(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);

        DdlPreparedPack.getInPackList(statusList, "workOrderStatus", packBean);
        return mesWorkOrderRDao.findByHqlWhere(packBean);
    }

    private MesMove createMove(MesPartSap mesPart, String source, String target, String org, String workCenterCode, double qty, String sn, Integer moveType) {
        MesMove move = new MesMove();
        if (!Objects.isNull(mesPart)) {
            move.setMeins(mesPart.getUnit());
        }
        move.setOrganizeCode(org);
        move.setFactoryCode(org);
        move.setLgort(source);
        move.setUmlgo(target);
        move.setMenge(qty);
        move.setPostDate(TimeTool.getToday());
        move.setProductSn(sn);
        move.setMoveType(moveType);
        move.setPostTime(TimeTool.getTimeShortWithColon());
        move.setWorkCenter(workCenterService.getErpWorkCenterCode(org, workCenterCode));
        ConvertBean.serviceModelInitialize(move, AuthUtil.getSessionUser().getUserName());
        return move;
    }
    @Override
    public List<MesProductionRecord> getPreDayReportMesProductionRecord(String organizeCode,List<MesShift> mesShiftList, String  workCenterCode ) {

        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode,"workCenterCode",ddlPackBean);
        DdlPreparedPack.getNumEqualPack( MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(),"reportStatus",ddlPackBean);
        //获取前一天的早班到当天早班的区间的所有加工记录，统计个数
        DdlPreparedPack.timeBuilder(
                new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(),1))+" 08:00:00",
                TimeTool.getToday()+" 08:00:00",
                "completeDateTime", ddlPackBean, true);

        return  productionRecordRao.findByHqlWhere(ddlPackBean);
    }
    @Override
    public List<MesProduceSn> getPreDayReportMesProduceSn(String organizeCode,List<MesProductionRecord> mesProductionRecordList ) {
        DdlPackBean snPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(mesProductionRecordList.stream().map(MesProductionRecord::getProductSn).collect(Collectors.toList()),
                "productSn",snPackBean);
        return mesProduceSnRao.findByHqlWhere(snPackBean);
    }
    @Override
    public List<MesWorkOrder> insertMesWorkOrder(List<MesProductionRecord> mesProductionRecordList,List<MesShift> mesShiftList, String organizeCode, String userName) {
        List<MesWorkOrder> mesWorkOrderList=new ArrayList<>();
        Map<String, List<MesProductionRecord>> mesProductionRecordListMap = mesProductionRecordList.stream().collect(Collectors.groupingBy(sn -> sn.getWorkCenterCode() +"="+ sn.getPartNo()));
        for (Map.Entry<String, List<MesProductionRecord>> stringListEntry : mesProductionRecordListMap.entrySet()) {
            List<MesProductionRecord> value = stringListEntry.getValue();
            MesProductionRecord mesProductionRecord = value.get(0);
            String[] split = stringListEntry.getKey().split("=");
            String workCenterCode = split[0];
            String partNo = split[1];
            //查询工作中心
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                MesException.throwMesBusiException("产线【%s】不存在",workCenterCode);
            }
            MesWorkOrder item=new MesWorkOrder();
            item.setOrganizeCode(organizeCode);
            item.setWorkCenterCode(workCenterCode);
            item.setErpWorkCenter(mesWorkCenter.getErpWorkCenter());
            item.setPartNo(partNo);
            item.setWorkOrderType(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue());
            item.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue());
            List<MesShift> collect = mesShiftList.stream().filter(s -> s.getShiftCode().equalsIgnoreCase("1")).collect(Collectors.toList());
            MesShift mesShift= collect.get(0);
            item.setShiftCode(mesShift.getShiftCode());
            item.setShiftCode(mesShift.getShiftName());
            String today = TimeTool.getToday();
            item.setPlanStartTime(today+" "+mesShift.getStartTime()+":00");
            item.setPlanEndTime(today+" "+mesShift.getEndTime()+":00");

            //生成工单号
            List<String> orderList=new ArrayList<>();
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel("MES_WORK_CENTER_ORDER_NO");
            genSerialNoModel.setPartNo(workCenterCode);
            orderList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(organizeCode), userName, organizeCode, 1).getResultList();

            String orderNo = orderList.get(0);
            DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(orderNo, "workOrderNo", orderPackBean);
            if (mesWorkOrderRDao.isExitByHql(orderPackBean)) {
                MesException.throwMesBusiException("单号流水码生成重复");
            }
            item.setWorkOrderNo(orderNo);
            item.setPartName(mesProductionRecord.getPartName());
            //生产时间新增取当天
            item.setProduceTime(TimeTool.getToday());
            item.setUnCompleteQty(0d);
            item.setQty(value.stream().mapToDouble(MesProductionRecord::getQty).sum());
            item.setReportedQty(item.getQty());
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());


            //校验标识
            //若工单类型为排序
            List<MesWorkOrderToWms> saveMesWorkOrderToWms = new ArrayList<>();
            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
                //校验产线类型和工单类型是否匹配
                if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue())) {
                    MesException.throwMesBusiException("产线【%s】类型为【%s】，请选择非排序产线", mesWorkCenter.getWorkCenterCode(),
                            MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
                }

                //校验物料生产版本是否存在
                MesProductVersion mesProductVersion = checkMesProductVersion(item);
                item.setProductVersion(mesProductVersion.getProductVersion());
                //获取bom信息
                List<MesBom> bomList =  findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());

                //发送工单信息给WMS
                saveMesWorkOrderToWms.addAll(saveMesWorkOrderToWms(item, bomList));
            }

            item.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            item.setFid(UUID.randomUUID().toString());
            ConvertBean.serviceModelInitialize(item, userName);
            MesWorkOrder mesWorkOrder = workOrderRepository.insert(item);
            //保存发送WMS数据
            if (CollectionUtils.isNotEmpty(saveMesWorkOrderToWms)) {
                mesWorkOrderToWmsRDao.saveAll(saveMesWorkOrderToWms);
            }
            saveMesWorkOrderLog(mesWorkOrder);
            mesWorkOrderList.add(mesWorkOrder);
        }

        return mesWorkOrderList;
    }

 
    public List<MesWorkOrderToWms> saveMesWorkOrderToWms(MesWorkOrder mesWorkOrder, List<MesBom> mesBomList) {
        List<MesWorkOrderToWms> mesWorkOrderToWmsList = new ArrayList<>();
        MesWorkOrderToWms mesWorkOrderToWms = null;
        for (MesBom mesBom : mesBomList) {
            mesWorkOrderToWms = new MesWorkOrderToWms();
            BeanUtils.copyProperties(mesWorkOrder, mesWorkOrderToWms, "id");
            mesWorkOrderToWms.setUnit(mesBom.getUnit());
            mesWorkOrderToWms.setItemPartNo(mesBom.getItemPartNo());
            mesWorkOrderToWms.setItemPartName(mesBom.getItemPartName());
            mesWorkOrderToWms.setItemUnit(mesBom.getItemUnit());
            mesWorkOrderToWms.setItemQty(MathOperation.mul(mesWorkOrder.getQty(), mesBom.getItemQty()));
            ConvertBean.serviceModelInitialize(mesWorkOrderToWms, mesWorkOrderToWms.getCreateUser());
            mesWorkOrderToWmsList.add(mesWorkOrderToWms);
        }
        return mesWorkOrderToWmsList;
    }

    public void doPcnJobProductReport(List<MesProductionRecord> mesProductionRecordList,List<MesShift> mesShiftList,String organizeCode, String userName ,MesWorkOrder mesWorkOrderDb) {

        //查询物料信息
        DdlPackBean ddlPackBeanPart = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getPartNo(), "partNo", ddlPackBeanPart);
        MesPart mesPart = mesPartRDao.getByProperty(ddlPackBeanPart);
        if (null == mesPart) {
            MesException.throwMesBusiException("物料【%s】信息不存在", mesWorkOrderDb.getPartNo());
        }

        //查询产线信息
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", mesWorkOrderDb.getWorkCenterCode());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrderDb);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms =  findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());

        //保存数据
//        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
//        for (MesBom mesBom : mesBoms) {
//            mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
//        }
//        mesProductOffLineService.insertBatch(mesProductOffLineList);

         //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        MesProductOffLine newMesProductOffLine;
        for (MesProductionRecord produceSn : mesProductionRecordList) {
            for (MesBom mesBom : mesBoms) {
                newMesProductOffLine = new MesProductOffLine();
                newMesProductOffLine.setReportPartNo(mesWorkOrderDb.getPartNo());
                newMesProductOffLine.setReportPartNameRdd(mesWorkOrderDb.getPartName());
                newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                newMesProductOffLine.setItemQty(mesBom.getItemQty());
                newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                newMesProductOffLine.setQty(1d);
                newMesProductOffLine.setReportSn(produceSn.getProductSn());
                newMesProductOffLine.setBomVersion(mesWorkOrderDb.getProductVersion());
                newMesProductOffLine.setSerialNumber(produceSn.getProductSn());
                newMesProductOffLine.setUnit(mesBom.getUnit());
                newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                newMesProductOffLine.setWorkOrderNo(mesWorkOrderDb.getWorkOrderNo());
                newMesProductOffLine.setWorkCenterCode(mesWorkOrderDb.getWorkCenterCode());
                newMesProductOffLine.setWorkCellCode(mesWorkOrderDb.getWorkCellCode());
                newMesProductOffLine.setReportType(mesWorkOrderDb.getReportType());
                newMesProductOffLine.setSapWorkCenter(mesWorkOrderDb.getErpWorkCenter());
                newMesProductOffLine.setOrganizeCode(organizeCode);

                ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                mesProductOffLineList.add(newMesProductOffLine);
            }
        }
        mesProductOffLineRDao.saveAll(mesProductOffLineList);
    }
    private void updateMesWorkOrder(MesWorkOrder mesWorkOrder, String userName ) {
        //修改工单，需要重新同步
        mesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        ConvertBean.serviceModelUpdate(mesWorkOrder, userName);
        workOrderRepository.update(mesWorkOrder);
        //保存记录
        saveMesWorkOrderLog(mesWorkOrder);
    }
    public void saveMesWorkOrderLog(MesWorkOrder mesWorkOrder) {
        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(mesWorkOrder, workOrderLog, "id");
        mesWorkOrderLogRepository.insert(workOrderLog);
    }
    public List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }
    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails =  findMesBomByPartNoAndBomVersion(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = MathOperation.div(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }
    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
        if (org.springframework.util.CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom next = mesBoms.iterator().next();
        //如果存在多个bomCode取最新的一个
        return mesBoms.stream().collect(Collectors.groupingBy(MesBom::getBomCode)).get(next.getBomCode());
    }


    private boolean checkWorkCenterPartRelation(String workCenterCode,String partNo){
        DdlPackBean partRelationPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode,"workCenterCode", partRelationPackBean);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", partRelationPackBean);
        int count = partRelationRao.findByHqlWhereCount(partRelationPackBean);
        return count > 0;
    }

}
