package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.report;

import cn.estsh.i3plus.ext.mes.api.base.IMesProduceSnService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.api.base.file.IMesFileService;
import cn.estsh.i3plus.ext.mes.api.busi.report.IMesYfReportService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesYfReportDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanMapUtilsExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesTraceabilityTemp;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportResultModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesProdRuleReportConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.report.MesTraceabilityReportDataModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesTraceabilityReportModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.ResultBean;
import jodd.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 报表-业务方法实现
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/15 11:31
 * @Modify:
 **/
@Service
@Slf4j
public class MesYfReportServiceImpl implements IMesYfReportService {

    @Autowired
    private IMesYfReportDao mesYfReportDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRDao;

    @Autowired
    private MesProdRuleSortCfgRepository mesProdRuleSortCfgRepository;

    @Autowired
    private MesProdRuleNoSortCfgRepository mesProdRuleNoSortCfgRepository;

    @Autowired
    private IMesProduceSnService mesProduceSnService;

    @Autowired
    private IMesProductionRecordService mesProductionRecordService;

    @Autowired
    private MesProductionAssemblyRepository mesProductionAssemblyRepository;

    @Autowired
    private MesWorkCellScanMonitorLogRepository workCellScanMonitorLogRepository;

    @Autowired
    private MesProcessRepository mesProcessRepository;

    @Autowired
    private MesWorkCenterRepository workCenterRepository;

    @Autowired
    private IMesFileService mesFileService;

    @Override
    public ListPager<MesDeviceDataQueryReportResultModel> queryMesDeviceDataByPager(MesDeviceDataQueryReportConditionModel queryReportModel, Pager pager) {
        ListPager<MesDeviceDataQueryReportResultModel> mesDeviceDataQueryReportResultModelListPager =
                mesYfReportDao.queryMesDeviceDataByPager(queryReportModel, pager);
        List<MesDeviceDataQueryReportResultModel> objectList = mesDeviceDataQueryReportResultModelListPager.getObjectList();
        for (MesDeviceDataQueryReportResultModel model : objectList) {
            model.setLogTypeName(MesExtEnumUtil.LOG_TYPE.valueOfDescription(model.getLogType()));
        }
        return new ListPager<>(objectList, mesDeviceDataQueryReportResultModelListPager.getObjectPager());
    }

    @Override
    public ResultBean queryMesProdRuleByPager(MesProdRuleReportConditionModel queryReportModel, Pager pager) {
        DdlPackBean queryPackBean = DdlPackBean.getDdlPackBean(queryReportModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(queryReportModel.getEquipmentCode(), "equipmentCode", queryPackBean);
        if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == queryReportModel.getIsSort()) {
            //查询“排序产品加工规则配置”
            DdlPreparedPack.getStringEqualPack(queryReportModel.getWorkCenterCode(), "workCenterCode", queryPackBean);
            DdlPreparedPack.getStringEqualPack(queryReportModel.getWorkCellCode(), "workCellCode", queryPackBean);
            pager = PagerHelper.getPager(pager, mesProdRuleSortCfgRepository.findByHqlWhereCount(queryPackBean));
            List<MesProdRuleSortCfg> mesProdRuleSortCfgList = mesProdRuleSortCfgRepository.findByHqlWherePage(queryPackBean, pager);
            return ResultBean.success("查询设备数据成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(new ListPager(mesProdRuleSortCfgList, pager));
        } else {
            //查询“非排序产品加工规则配置”
            DdlPreparedPack.getStringEqualPack(queryReportModel.getInPartNo(), "inPartNo", queryPackBean);
            DdlPreparedPack.getStringEqualPack(queryReportModel.getInPartNoRule(), "inPartNoRule", queryPackBean);
            DdlPreparedPack.getStringEqualPack(queryReportModel.getOutPartNo(), "outPartNo", queryPackBean);
            pager = PagerHelper.getPager(pager, mesProdRuleNoSortCfgRepository.findByHqlWhereCount(queryPackBean));
            List<MesProdRuleNosortCfg> mesProdRuleNosortCfgList = mesProdRuleNoSortCfgRepository.findByHqlWherePage(queryPackBean, pager);
            return ResultBean.success("查询设备数据成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(new ListPager(mesProdRuleNosortCfgList, pager));
        }
    }


    @Override
    public ListPager<MesProduceSn> queryMesSuspiciousSnMsgByPager(MesProduceSn bean, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getQcStatus(), "qcStatus", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getProductSn(), "productSn", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getLotNo(), "lotNo", packBean);
        if (!StringUtil.isEmpty(bean.orderBy())) {
            packBean.setOrderByStr(bean.orderBy());
        } else {
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        }
        pager = PagerHelper.getPager(pager, mesProduceSnRDao.findByHqlWhereCount(packBean));

        List<MesProduceSn> resultList = mesProduceSnRDao.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    @Override
    public List<MesTraceabilityReportModel> findTraceabilityReport(MesProduceSn mesProduceSn) {
        DdlPackBean packBean = getDdlPackBean(mesProduceSn);
        //查询条码记录表
        MesProduceSn produceSn = mesProduceSnService.findMesProduceSn(mesProduceSn.getOrganizeCode(), mesProduceSn.getProductSn()).get(0);
        //设备加工记录表
        List<MesProductionRecord> productionRecordList = mesProductionRecordService.findMesProductionRecord(mesProduceSn.getOrganizeCode(), mesProduceSn.getProductSn());
        List<String> processCodeList = productionRecordList.stream().map(MesProductionRecord::getProcessCode).filter(StringUtil::isNotEmpty).collect(Collectors.toList());
        //装配记录表
        List<MesProductionAssembly> productionAssemblyList = mesProductionAssemblyRepository.findByHqlWhere(packBean);
        Map<String, List<MesProductionAssembly>> productionAssemblyMap = CollectionUtils.isEmpty(productionAssemblyList) ? null : productionAssemblyList.stream().collect(Collectors.groupingBy(t -> t.getWorkCenterCode() + t.getWorkCellCode() + t.getEquipmentCode()));
        //工位扫描监控日志
        List<MesWorkCellScanMonitorLog> mesWorkCellScanMonitorLogs = getWorkCellScanMonitorLogList(productionRecordList);
        Map<String, List<MesWorkCellScanMonitorLog>> mesWorkCellScanMonitorMap = CollectionUtils.isEmpty(mesWorkCellScanMonitorLogs) ? null : mesWorkCellScanMonitorLogs.stream().collect(Collectors.groupingBy(t -> t.getWorkCenterCode() + t.getWorkCellCode() + t.getEquipmentCode()));
        //查询工序名称
        Map<String, MesProcess> processMap = getMesProcessMap(mesProduceSn, processCodeList);
        //数据组装
        List<MesTraceabilityReportModel> mesTraceabilityReportModelList = new ArrayList<>();
        MesTraceabilityReportModel mesTraceabilityReportModel = new MesTraceabilityReportModel();
        BeanUtils.copyProperties(produceSn, mesTraceabilityReportModel);
        mesTraceabilityReportModel.setProdType(getWorkCenter(produceSn).getCenterType());
        int duration = 0;
        for (MesProductionRecord mesProductionRecord : productionRecordList) {
            MesTraceabilityReportDataModel dataModel = new MesTraceabilityReportDataModel();
            BeanUtils.copyProperties(mesProductionRecord, dataModel);
            //持续时间
            if (StringUtil.isNotEmpty(dataModel.getStartDateTime()) && StringUtil.isNotEmpty(dataModel.getCompleteDateTime())) {
                dataModel.setDuration(getTimeDifference(dataModel.getStartDateTime(), dataModel.getCompleteDateTime()));
                dataModel.setDurationStr(dateFormat(dataModel.getDuration()));
                duration += dataModel.getDuration();
            }
            //装配记录表
            String key = mesProductionRecord.getWorkCenterCode() + mesProductionRecord.getWorkCellCode() + mesProductionRecord.getEquipmentCode();
            if (!Objects.isNull(productionAssemblyMap) && productionAssemblyMap.containsKey(key)) {
                dataModel.setProductionAssemblyList(productionAssemblyMap.get(key));
            }
            //工位扫描监控日志
            if (!Objects.isNull(mesWorkCellScanMonitorMap) && mesWorkCellScanMonitorMap.containsKey(key)) {
                dataModel.setMesWorkCellScanMonitorLogs(mesWorkCellScanMonitorMap.get(key));
            }
            //查询工序名称
            if(StringUtil.isNotEmpty(dataModel.getProcessCode())){
                if(!Objects.isNull(processMap) && processMap.containsKey(dataModel.getProcessCode())){
                    dataModel.setProcessName(processMap.get(dataModel.getProcessCode()).getProcessName());
                }else{
                    dataModel.setProcessName(dataModel.getProcessCode());
                }
            }

            mesTraceabilityReportModel.getMesTraceabilityReportDataModelList().add(dataModel);
        }
        //开始时间
        mesTraceabilityReportModel.setProduceBegin(productionRecordList.iterator().next().getStartDateTime());
        //生产状态
        mesTraceabilityReportModel.setProdStatus(mesTraceabilityReportModel.getSnStatus());
        //结束时间
        if (mesTraceabilityReportModel.getSnStatus() >= MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue()) {
            mesTraceabilityReportModel.setProdStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue());
            mesTraceabilityReportModel.setProduceEnd(productionRecordList.get(productionRecordList.size() - 1).getCompleteDateTime());
        }
        mesTraceabilityReportModel.setDurationStr(dateFormat(duration));
        mesTraceabilityReportModel.setDuration(duration);

        //生成PDF文件并上传
        try {
            mesTraceabilityReportModel.setUrl(MesTraceabilityTemp.getMesTraceabilityTemp(mesFileService,mesTraceabilityReportModel));
        } catch (IOException e) {
            MesException.throwMesBusiException("删除文件：生成PDF文件失败");
        }
        mesTraceabilityReportModelList.add(mesTraceabilityReportModel);


        return mesTraceabilityReportModelList;
    }

    private MesWorkCenter getWorkCenter(MesProduceSn produceSn) {
        MesWorkCenter workCenter = workCenterRepository.getByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID, MesExtConstWords.WORK_CENTER_CODE},
                new Object[]{produceSn.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), produceSn.getWorkCenterCode()});
        if (null == workCenter) MesException.throwMesBusiException("条码[%s]关联的生产线[%s]信息不存在", produceSn.getProductSn(), produceSn.getWorkCenterCode());
        return workCenter;
    }

    private List<MesWorkCellScanMonitorLog> getWorkCellScanMonitorLogList(List<MesProductionRecord> productionRecordList) {
        if (CollectionUtils.isEmpty(productionRecordList)) return null;
        List<MesWorkCellScanMonitorLog> workCellScanMonitorLogList = new ArrayList<>();
        for (MesProductionRecord productionRecord : productionRecordList) {
            if (null == productionRecord) continue;
            //云端 MesWorkCellScanMonitorLog 按照 工厂+生产线 分表  只能根据开模ID查询, 条码字段是产出条码 不是主条码, 一模多腔场景存的是多个条码分号分隔
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(productionRecord.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(productionRecord.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
            DdlPreparedPack.getStringEqualPack(productionRecord.getWorkCellCode(), MesExtConstWords.WORK_CELL_CODE, packBean);
            DdlPreparedPack.getNumEqualPack(productionRecord.getMouldRecordId(), MesExtConstWords.MOULD_RECORD_ID, packBean);
            List<MesWorkCellScanMonitorLog> itemList = workCellScanMonitorLogRepository.findByHqlWhere(packBean);
            if (!CollectionUtils.isEmpty(itemList)) workCellScanMonitorLogList.addAll(itemList);
        }
        return workCellScanMonitorLogList;
    }

    private Map<String, MesProcess> getMesProcessMap(MesProduceSn mesProduceSn, List<String> processCodeList) {
        Map<String, MesProcess> processMap = null;
        if(!CollectionUtils.isEmpty(processCodeList)){
            try {
                processMap = BeanMapUtilsExt.getBeanMap(mesProcessRepository, "processCode", mesProduceSn.getOrganizeCode(), processCodeList, "工序");
            } catch (Exception e) {
                log.info(e.getMessage());
            }
        }
        return processMap;
    }

    private DdlPackBean getDdlPackBean(MesProduceSn mesProduceSn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesProduceSn.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSn.getProductSn(), "productSn", packBean);
        DdlPreparedPack.getOrderBy(null, null, packBean);
        return packBean;
    }

    public static String dateFormat(int secound) {
        StringBuilder time = new StringBuilder();
        SimpleDateFormat destFormat = new SimpleDateFormat(DateUtil.SHORT_FORMAT_HOUR_MINUTE);
        try {
            // 将时间差转换为小时、分钟和秒
            long diffSeconds = (secound) % 60;
            long diffMinutes = (secound / 60) % 60;
            long diffHours = secound / (60 * 60);
            return destFormat.format(destFormat.parse(time.append(diffHours).append(":").append(diffMinutes).append(":").append(diffSeconds).toString()));
        } catch (ParseException var17) {
            var17.printStackTrace();
            return null;
        }
    }

    public int getTimeDifference(String startTime, String endTime) {
        try {
            return TimeTool.getSecoundsBetweenTime(1, startTime, endTime);
        } catch (ParseException var17) {
            var17.printStackTrace();
            return 0;
        }
    }
}
